"""
The Ramer-Douglas-Peucker algorithm roughly ported from the pseudo-code provided
by http://en.wikipedia.org/wiki/Ramer-Douglas-Peucker_algorithm
Implementation: https://github.com/sebleier/RDP/blob/master/__init__.py
MIT License
"""
from math import sqrt

def run_rdp(x1d_np, y1d_np, epsilon):
    import numpy as np
    merged_list = tuple(zip(x1d_np.tolist(), y1d_np.tolist()))
    results = rdp(merged_list, epsilon)
    x_new, y_new = zip(*results)
    return np.asarray(x_new), np.asarray(y_new)

def distance(a, b):
    return  sqrt((a[0] - b[0]) ** 2 + (a[1] - b[1]) ** 2)


def point_line_distance(point, start, end):
    if (start == end):
        return distance(point, start)
    else:
        n = abs(
            (end[0] - start[0]) * (start[1] - point[1]) -
            (start[0] - point[0]) * (end[1] - start[1])
        )
        d = sqrt(
            (end[0] - start[0]) ** 2 + (end[1] - start[1]) ** 2
        )
        return n / d


def rdp(points, epsilon):
    """Reduces a series of points to a simplified version that loses detail, but
    maintains the general shape of the series.
    """
    dmax = 0.0
    index = 0
    for i in range(1, len(points) - 1):
        d = point_line_distance(points[i], points[0], points[-1])
        if d > dmax:
            index = i
            dmax = d

    if dmax >= epsilon:
        results = rdp(points[:index+1], epsilon)[:-1] + rdp(points[index:], epsilon)
    else:
        results = [points[0], points[-1]]

    return results


from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from emda.iotools import write_mrc,read_map
from emda.quaternions import get_quaternion,get_RM
from emda.mapfit.utils import double_the_axes, get_FRS
from emda.config import *

def write_mrc2(mapdata,filename,unit_cell,map_origin):
    import mrcfile as mrc
    import numpy as np
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(mapdata))))
    # removing outer regions
    nx, ny, nz = data2write.shape
    assert nx == ny == nz
    dx = int(nx/4); xx = int(nx/2)
    newdata = data2write[dx:dx+xx,dx:dx+xx,dx:dx+xx]
    file = mrc.new(name=filename, data=np.float32(newdata), compression=None, overwrite=True)
    file.header.cella.x = unit_cell[0]
    file.header.cella.y = unit_cell[1]
    file.header.cella.z = unit_cell[2]
    file.header.nxstart = map_origin[0]
    file.header.nystart = map_origin[1]
    file.header.nzstart = map_origin[2]
    file.close()

def map_transform(mapname,t,r,ax,outname='translformed.mrc'):
    import emda.iotools
    import numpy as np
    from emda.quaternions import get_quaternion,get_RM, rotationMatrixToEulerAngles
    import fcodes_fast
    from emda.mapfit import utils
    from scipy.ndimage.interpolation import shift
    uc, arr, origin = emda.iotools.read_map(mapname)
    '''theta = [ax,r]
    q = get_quaternion(theta)
    q = q/np.sqrt(np.dot(q,q))
    rotmat = get_RM(q)
    rotmat = np.transpose(rotmat)
    print('rotation matrix:', rotmat)
    print('Applied rotation in degrees [Euler angles]: ', 
           rotationMatrixToEulerAngles(rotmat) * 180./np.pi)
    print('Applied rotation in degrees [Overall]: ',
           np.arccos((np.trace(rotmat) - 1)/2) * 180./np.pi)
    print('Applied translation in Angstrom: ', t)
    hf = np.fft.fftshift(np.fft.fftn(arr))
    cx,cy,cz = hf.shape
    if np.sqrt(np.dot(t, t)) < 1.0e-3:
        st = 1.0
    else:
        t = (np.asarray(t)) / uc[:3]
        st,_,_,_ = fcodes_fast.get_st(cx,cy,cz,t)
    print('Applied translation in Angstrom: ', t * uc[:3])
    transformed_map = np.real(
                              np.fft.ifftn(
                              np.fft.ifftshift(
                              get_FRS(uc,rotmat,hf*st)[:,:,:,0])))
    '''
    # real space map interpolation - fcodes_fast
    t_frac = utils.ang2frac(t,uc[0]/arr.shape[0])
    theta = [ax,r]
    q = get_quaternion(theta)
    q = q/np.sqrt(np.dot(q,q))
    rotmat = get_RM(q)
    #rotmat = np.transpose(rotmat)
    print('rotation matrix:', rotmat)
    print('Applied rotation in degrees [Euler angles]: ', 
           rotationMatrixToEulerAngles(rotmat) * 180./np.pi)
    print('Applied rotation in degrees [Overall]: ',
           np.arccos((np.trace(rotmat) - 1)/2) * 180./np.pi)
    print('Applied translation in Angstrom: ', t)
    nx, ny,nz = arr.shape
    transformed_map = fcodes_fast.trilinear_map(rotmat,arr,nx,ny,nz)
    transformed_map = shift(transformed_map, t_frac)
    '''# using scipy - rotate function
    from scipy import ndimage
    f = np.fft.fftshift(np.fft.fftn(arr))
    frs_real = ndimage.rotate(f.real, r, axes=(1,2), reshape=False)
    frs_imag = ndimage.rotate(f.imag, r, axes=(1,2), reshape=False)
    frs = frs_real + 1j*frs_imag
    transformed_map = np.real(np.fft.ifftn(np.fft.ifftshift(frs)))
    angle_rad = np.deg2rad(r)
    c, s = np.cos(angle_rad), np.sin(angle_rad)
    rot_matrix = np.array([[c, s],
                              [-s, c]])
    print('rotation matrix:', rot_matrix)'''
    emda.iotools.write_mrc(transformed_map,outname,uc,origin)
    return transformed_map

if (__name__ == "__main__"):
    mapname = '/Users/ranganaw/MRC/REFMAC/Bianka/fit/com_at_box_center/using_normalised_sf/shifted_to/static_map.mrc'
    uc,arr,origin = read_map(mapname)
    theta_init=[(0,0,1),180.0]
    q = get_quaternion(theta_init)
    q = q/np.sqrt(np.dot(q,q))
    rotmat = get_RM(q)
    arr = double_the_axes(arr)
    hf = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr)))
    frt_full = get_FRS(uc,rotmat,hf)[:,:,:,0]
    write_mrc2(frt_full,'staticmap_z_180deg.mrc',uc,origin)


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from emda.config import *

def test():
    print('iotools test ... Passed')

def read_map(mapname11):
    # Reading .map/mrc file into a numpy 3d array
    import mrcfile as mrc
    file11 = mrc.open(mapname11)
    cell1 = np.array(file11.header.cella)
    a1 = cell1['x']; b1 = cell1['y']; c1 = cell1['z']
    uc = np.asarray([a1 * 1.0, b1 * 1.0, c1 * 1.0, 90.0, 90.0, 90.0])
    origin = [  1 * file11.header.nxstart,
                1 * file11.header.nystart,
                1 * file11.header.nzstart   ]
    ar11 = np.asarray(file11.data,dtype='float')
    file11.close()
    print(mapname11,ar11.shape,uc[:3])
    return uc,ar11,origin

def read_mtz(mtzfile):
    # Reading mtz file using gemmi
    import gemmi
    import numpy as np
    mtz = gemmi.read_mtz_file(mtzfile)
    uc = np.zeros(6, dtype='float')
    uc[0] = mtz.dataset(0).cell.a
    uc[1] = mtz.dataset(0).cell.b
    uc[2] = mtz.dataset(0).cell.c
    uc[3:] = 90.0
    # h - slowest; l - fastest
    '''l = np.array(mtz.column_with_label('H'), copy=False)
        k = np.array(mtz.column_with_label('K'), copy=False)
        h = np.array(mtz.column_with_label('L'), copy=False)
        ampl = np.array(mtz.column_with_label('Fout0'), copy=False)
        phas = np.array(mtz.column_with_label('Pout0'), copy=False)'''
    import pandas
    all_data = np.array(mtz, copy=False)
    df = pandas.DataFrame(data=all_data, columns=mtz.column_labels())
    # Index df by column_labels e.g. df['Fout0']
    return uc,df

def write_mrc(mapdata,filename,unit_cell,map_origin=[0.0,0.0,0.0], factor=1.0):
    # Writing out numpy 3D array data into mrc file
    # Note that mapdata is already density values.
    import mrcfile as mrc
    import numpy as np
    #data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(mapdata))))
    file = mrc.new(name=filename, data=np.float32(mapdata), compression=None, overwrite=True)
    file.header.cella.x = unit_cell[0] * factor
    file.header.cella.y = unit_cell[1] * factor
    file.header.cella.z = unit_cell[2] * factor
    file.header.nxstart = map_origin[0]
    file.header.nystart = map_origin[1]
    file.header.nzstart = map_origin[2]
    file.close()

def write_3d2mtz(uc,arr,outfile='output.mtz'):
    # Writing out numpy 3D day into mtz file
    # Fortran function e.g. prepare_hkl is used to
    # generate only hemisphere data
    import numpy as np
    import fcodes_fast
    import gemmi
    print(arr.shape)
    nx, ny, nz = arr.shape
    mtz = gemmi.Mtz()
    mtz.spacegroup = gemmi.find_spacegroup_by_name('P 1')
    mtz.cell.set(uc[0], uc[1], uc[2], 90, 90, 90)
    mtz.add_dataset('HKL_base')
    for label in ['H', 'K', 'L']: mtz.add_column(label, 'H')
    mtz.add_column('Fout0','F')
    mtz.add_column('Pout0','P')
    # calling fortran function  
    h,k,l,ampli,phase = fcodes_fast.prepare_hkl(arr,debug_mode,nx,ny,nz)
    nrows = len(h)
    ncols = 5 # Change this according to what columns to write
    data = np.ndarray(shape=(nrows,ncols), dtype=object)
    data[:,0] = -l.astype(int)
    data[:,1] = -k.astype(int)
    data[:,2] = -h.astype(int)
    data[:,3] = ampli.astype(np.float32)
    data[:,4] = phase.astype(np.float32)
    print(data.shape)
    mtz.set_data(data)
    mtz.write_to_file(outfile)

'''def write_3d2mtz_refmac(uc,arr1,arr2,outfile='output.mtz'):
    # One time function for REFMAC 3D refinement data
    import numpy as np
    import gemmi
    import fcodes_fast
    assert arr1.shape == arr2.shape
    nx, ny, nz = arr1.shape
    mtz = gemmi.Mtz()
    mtz.spacegroup = gemmi.find_spacegroup_by_name('P 1')
    mtz.cell.set(uc[0], uc[1], uc[2], 90, 90, 90)
    mtz.add_dataset('HKL_base')
    for label in ['H', 'K', 'L']: mtz.add_column(label, 'H')
    mtz.add_column('Fout0','F')
    mtz.add_column('Pout0','P')
    mtz.add_column('SIGF','Q')
    # calling fortran function
    h,k,l,ampli,phase = fcodes_fast.prepare_hkl(arr1,1,nx,ny,nz) # map
    _,_,_,ampli_noise,_ = fcodes_fast.prepare_hkl(arr2,1,nx,ny,nz) # map
    assert len(ampli) == len(ampli_noise)
    nrows = len(h)
    ncols = 6 # Change this according to what columns to write
    data = np.ndarray(shape=(nrows,ncols), dtype=object)
    data[:,0] = -l.astype(int)
    data[:,1] = -k.astype(int)
    data[:,2] = -h.astype(int)
    data[:,3] = ampli.astype(np.float32)
    data[:,4] = phase.astype(np.float32)
    data[:,5] = ampli_noise.astype(np.float32)
    print(data.shape)
    mtz.set_data(data)
    mtz.write_to_file(outfile)'''

def write_3d2mtz_refmac(uc,sgrid,f,f_noise,bfac,outfile='output.mtz'):
    # One time function for REFMAC 3D refinement data
    import numpy as np
    import gemmi
    import fcodes_fast
    assert f.shape == f_noise.shape
    nx, ny, nz = f.shape
    mtz = gemmi.Mtz()
    mtz.spacegroup = gemmi.find_spacegroup_by_name('P 1')
    mtz.cell.set(uc[0], uc[1], uc[2], 90, 90, 90)
    mtz.add_dataset('HKL_base')
    for label in ['H', 'K', 'L']: mtz.add_column(label, 'H')
    #mtz.add_column('Fout0','F')
    for label in ['Fout'+str(int(i)) for i in bfac]: mtz.add_column(label,'F')
    #mtz.add_column('SIGF','Q')
    for label in ['SigF'+str(int(i)) for i in bfac]: mtz.add_column(label,'Q')
    mtz.add_column('Pout0','P')
    # calling fortran function
    h,k,l,ampli,noise,phase = fcodes_fast.prepare_hkl_bfac(sgrid,f,f_noise,bfac,1,nx,ny,nz,len(bfac)) # map
    nrows = ampli.shape[0]
    ncols = 4 + 2 * ampli.shape[1] # Change this according to what columns to write
    data = np.ndarray(shape=(nrows,ncols), dtype=object)
    data[:,0] = -l.astype(int)
    data[:,1] = -k.astype(int)
    data[:,2] = -h.astype(int)
    for i in range(ampli.shape[1]):
        data[:,i+3] = ampli[:,i].astype(np.float32)
        data[:,i+3+ampli.shape[1]] = noise[:,i].astype(np.float32)
    data[:,-1] = phase.astype(np.float32)
    print(data.shape)
    mtz.set_data(data)
    mtz.write_to_file(outfile)

def write_mtz2_3d(h,k,l,f,nx,ny,nz):
    # Output mtz data into numpy 3D array
    # NEED CAREFUL TEST
    import numpy as np
    arr = np.zeros((nx*ny*nx), dtype=np.complex)
    xv, yv, zv = np.meshgrid(h,k,l)
    xvf = xv.flatten(order='F')
    lb = (len(arr)-len(xvf))//2
    ub = lb + len(xvf)
    print(lb,ub)
    arr[lb:ub] = f
    f3d = arr.reshape(nx,ny,nz)
    mapdata = np.fft.ifftn(np.fft.fftshift(f3d))
    return mapdata

def write_mtz2_3d_gemmi(mtzfile,map_size):
    # Writing out mtz data into ccp4.map format
    # Note that write_ccp4_map only supports numbers with factors 2, 3 and 5
    import gemmi
    mtz = gemmi.read_mtz_file(mtzfile)
    ccp4 = gemmi.Ccp4Map()
    ccp4.grid = mtz.transform_f_phi_to_map('Fout0', 'Pout0', map_size)
    ccp4.update_ccp4_header(2, True)
    ccp4.write_ccp4_map('output.ccp4')
    return

def resample2staticmap(target_pix_size,target_dim,uc,arr,fobj=None):
    # Resamling arr into an array having target_dim
    import numpy as np
    tnx,tny,tnz = target_dim
    assert uc[0] == uc[1] == uc[2]
    nx, ny, nz = arr.shape
    tnx, tny, tnz = target_dim
    assert tnx == tny == tnz
    assert nx == ny == nz
    current_pix_size = round(uc[0]/nx,3)
    target_pix_size = round(target_pix_size,3)
    print('pixel size [current, target]: ', current_pix_size,target_pix_size)
    if fobj is not None:
        fobj.write('pixel size [current, target]: '+
                    str(current_pix_size)+' '
                    +str(target_pix_size)+' \n')
    new_arr = resample(arr, target_dim)
    '''if current_pix_size == target_pix_size:
        sf = False
        if tnx == nx:
            print('No change of dims')
            if fobj is not None:
                fobj.write('No change of dims \n')
            new_arr = arr
        elif nx < tnx:
            # pad with zeros 
            print('Padded with zeros')
            if fobj is not None:
                fobj.write('Padded with zeros \n')
            dx = abs(tnx - nx)//2
            new_arr = np.zeros((tnx,tny,tnz),dtype='float')
            new_arr[dx:nx+dx, dx:nx+dx, dx:nx+dx] = arr
        elif tnx < nx:
            # strip out edge rows 
            print('Cropped image')
            if fobj is not None:
                fobj.write('Cropped image \n')
            dx = abs(nx - tnx)//2
            new_arr = np.zeros((target_dim),dtype='float')
            new_arr = arr[dx:dx+tnx,dx:dx+tnx,dx:dx+tnx]
    elif current_pix_size != target_pix_size:
        sf = True
        print('Resizing in Fourier space and transforming back')
        if fobj is not None:
            fobj.write('Resizing in Fourier space and transforming back \n')
        #new_arr = np.real(np.fft.ifftn(np.fft.fftn(arr,s=target_dim)))
        new_arr = resample(arr, target_dim)
    if sf:
        return [new_arr, f_new_arr]
    else:
        return [new_arr]'''
    return new_arr

def resample(x, num):
    import numpy as np
    # Check if dims are even
    if x.shape[0] % 2 != 0:
        xshape = list(x.shape)
        xshape[0] = xshape[0] + 1
        xshape[1] = xshape[1] + 1
        xshape[2] = xshape[2] + 1
        temp = np.zeros(xshape, x.dtype)
        temp[:-1, :-1, :-1] = x
        x = temp
    # Forward transform
    X = np.fft.fftn(x)
    X = np.fft.fftshift(X)
    # Placeholder array for output spectrum
    newshape = list(x.shape)
    newshape[0] = num[0] 
    newshape[1] = num[1]
    newshape[2] = num[2]
    if num[0] % 2 != 0:
        newshape[0] = num[0] + 1 
        newshape[1] = num[1] + 1
        newshape[2] = num[2] + 1 
    Y = np.zeros(newshape, X.dtype)
    # no-sampling
    if X.shape[0] == newshape[0]:
        Y[:, :, :] = X
    # upsampling
    if X.shape[0] < newshape[0]:
        dx = abs(newshape[0] - X.shape[0]) // 2
        Y[dx:dx+X.shape[0],
          dx:dx+X.shape[0],
          dx:dx+X.shape[0]] = X
    # downsampling
    if newshape[0] < X.shape[0]:
        dx = abs(newshape[0] - X.shape[0]) // 2
        Y[:, :, :] = X[dx:dx+newshape[0],
                       dx:dx+newshape[0],
                       dx:dx+newshape[0]]
    Y = np.fft.ifftshift(Y)
    return (np.fft.ifftn(Y)).real

def read_mmcif(mmcif_file):
    # Reading mmcif using gemmi and output as numpy 1D arrays
    from gemmi import cif
    import numpy as np
    doc = cif.read_file(mmcif_file)
    block = doc.sole_block() # cif file as a single block
    a = block.find_value('_cell.length_a')
    b = block.find_value('_cell.length_b')
    c = block.find_value('_cell.length_c')
    alf = block.find_value('_cell.angle_alpha')
    bet = block.find_value('_cell.angle_beta')
    gam = block.find_value('_cell.angle_gamma')
    cell = np.array([a,b,c,alf,bet,gam],dtype='float')
    # Reading X coordinates in all atoms
    col_x = block.find_values('_atom_site.Cartn_x')
    col_y = block.find_values('_atom_site.Cartn_y')
    col_z = block.find_values('_atom_site.Cartn_z')
    # Reading B_iso values
    col_Biso = block.find_values('_atom_site.B_iso_or_equiv')
    # Casting gemmi.Columns into a numpy array
    #xyz_np = np.zeros((3,len(col_x)), dtype='float')
    #xyz_np = np.zeros((3,len(col_x)), dtype='float')
    #xyz_np = np.zeros((3,len(col_x)), dtype='float')
    #xyz_np[0,:] = np.array(col_x, dtype='float', copy=False)
    #xyz_np[1,:] = np.array(col_y, dtype='float', copy=False)
    #xyz_np[2,:] = np.array(col_z, dtype='float', copy=False)
    x_np = np.array(col_x, dtype='float', copy=False)
    y_np = np.array(col_y, dtype='float', copy=False)
    z_np = np.array(col_z, dtype='float', copy=False)
    Biso_np = np.array(col_Biso, dtype='float', copy=False)
    return cell,x_np,y_np,z_np,Biso_np

def run_refmac_sfcalc(filename,resol,bfac):
    import os
    import os.path
    import subprocess
    # Creating the sfcalc.inp with custom parameters (resol, Bfac)
    sfcalc_inp = open("sfcalc.inp","w+")
    sfcalc_inp.write("mode sfcalc\n")
    sfcalc_inp.write("sfcalc cr2f\n")
    sfcalc_inp.write("resolution %f\n"%resol)
    sfcalc_inp.write("temp set %f\n"%bfac)
    sfcalc_inp.write("source em mb\n")
    sfcalc_inp.write("make hydrogen no\n")
    sfcalc_inp.write("end")
    sfcalc_inp.close()
    # Read in sfcalc_inp
    fmtz = filename[:-4]+".mtz"
    #cmd = ["f2mtz","HKLIN",filename,"HKLOUT",fmtz]
    cmd = ["refmac5","XYZIN",filename,"HKLOUT",fmtz]
    PATH='sfcalc.inp'
    logf = open("sfcalc.log", "w+")
    if os.path.isfile(PATH) and os.access(PATH, os.R_OK):
        print("sfcalc.inp exists and is readable")
        inp = open("sfcalc.inp","r")
        # Run the command with parameters from file f2mtz.inp
        subprocess.call(cmd,stdin=inp,stdout=logf)
    else:
        print("Either the file is missing or not readable")
    logf.close()
    inp.close()

def read_atomsf(atm, fpath=None):
    # outputs A and B coefficients corresponding to atom(atm)
    found = False
    with open(fpath) as myFile:
        for num, line in enumerate(myFile, 1):
            if line.startswith(atm):
                found = True
                #print('found at line:', num)
                break
                
    if not found:
        import sys 
        print(atm, 'is not found!')
        sys.exit('Program termination')
                
    f = open(fpath)
    all_lines = f.readlines()
    A = []
    B = []
    for i in range(4):
        if i == 0: 
            A.append(all_lines[num + i].split()[-1])
            B.append(0.0)
        elif i == 1:
           A =  A + all_lines[num + i].split()
        elif i == 2:
           B =  B + all_lines[num + i].split()    
    f.close()
    return np.asarray(A, dtype=np.float), np.asarray(B, dtype=np.float)


def pdb2mmcif(filename_pdb):
    import gemmi
    structure = gemmi.read_structure(filename_pdb)
    mmcif = structure.make_mmcif_document()
    mmcif.write_file('out.cif')

def mask_by_value(array,masking_value=0.0,filling_value=0.0):
    # Array values less_or_equal to masking_value are filled with
    # given filling value
    import numpy.ma as ma
    array_ma = ma.masked_less_equal(array, masking_value)
    array_masked_filled = array_ma.filled(filling_value)
    return array_masked_filled

def mask_by_value_greater(array,masking_value=0.0,filling_value=0.0):
    # Array values greater than masking_value are filled with
    # given filling value
    import numpy.ma as ma
    array_ma = ma.masked_greater(array, masking_value)
    array_masked_filled = array_ma.filled(filling_value)
    return array_masked_filled


##### below function are not frequently used.#####
def read_mrc(mapname11):
    import mrcfile as mrc
    file11 = mrc.open(mapname11)
    cell1 = np.array(file11.header.cella)
    a1 = cell1['x']; b1 = cell1['y']; c1 = cell1['z']
    uc = np.asarray([a1 * 1.0, b1 * 1.0, c1 * 1.0, 90.0, 90.0, 90.0])
    origin = [  1 * file11.header.nxstart,
              1 * file11.header.nystart,
              1 * file11.header.nzstart   ]
    ar11 = np.asarray(file11.data,dtype='float')
    ar11_centered = np.fft.fftshift(ar11) # CENTERED IMAGE
    nx,ny,nz = ar11_centered.shape
    print(mapname11,nx,ny,nz,uc[:3])
    file11.close()
    hf11 = np.fft.fftshift(np.fft.fftn(ar11_centered)) # CENTERED FFT OF CENTERED IMAGE
    return uc,hf11,origin

def write_3d2mtz_full(uc,arr,outfile='output.mtz'):
    # Write numpy 3D array into MTZ file.
    # Issue: It write out full sphere data
    import numpy.fft as fft
    print(arr.shape)
    nx, ny, nz = arr.shape
    import gemmi
    mtz = gemmi.Mtz()
    mtz.spacegroup = gemmi.find_spacegroup_by_name('P 1')
    mtz.cell.set(uc[0], uc[1], uc[2], 90, 90, 90)
    mtz.add_dataset('HKL_base')
    for label in ['H', 'K', 'L']: mtz.add_column(label, 'H')
    mtz.add_column('Fout0','F')
    mtz.add_column('Pout0','P')
    # Add more columns if you need
    x = fft.fftshift(fft.fftfreq(arr.shape[0])*nx)
    y = fft.fftshift(fft.fftfreq(arr.shape[1])*ny)
    z = fft.fftshift(fft.fftfreq(arr.shape[2])*nz)
    xv, yv, zv = np.meshgrid(x,y,z)
    xvf = xv.flatten(order='F')
    yvf = yv.flatten(order='F')
    zvf = zv.flatten(order='F')
    
    arr_real = np.real(arr)
    arr_imag = np.imag(arr)
    ampli = np.sqrt(np.power(arr_real,2) + np.power(arr_imag,2))
    phase = np.arctan2(arr_imag, arr_real) * 180 / np.pi
    
    ampli_1d = ampli.flatten(order='F')
    phase_1d = phase.flatten(order='F')
    
    nrows = len(xvf)
    ncols = 5 # Change this according to what columns to write
    data = np.ndarray(shape=(nrows,ncols), dtype=object)
    '''data[:,0] = -1 * zvf.astype(int)
        data[:,1] = -1 * xvf.astype(int)
        data[:,2] = -1 * yvf.astype(int)'''
    data[:,0] = zvf.astype(int)
    data[:,1] = xvf.astype(int)
    data[:,2] = yvf.astype(int)
    data[:,3] = ampli_1d.astype(np.float32)
    data[:,4] = phase_1d.astype(np.float32)
    print(data.shape)
    mtz.set_data(data)
    mtz.write_to_file(outfile)



"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology

This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

import fcodes_fast



"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import fcodes_fast
from .config import *

#debug_mode = 1 # 0: no debug info, 1: debug

def halfmaps_fsc_variance(hf1,hf2,bin_idx,nbin):
    assert hf1.shape == hf2.shape
    nx,ny,nz = hf1.shape
    fo,eo,noisevar,signalvar,totalvar,bin_fsc,bincount = fcodes_fast.calc_fsc_using_halfmaps(hf1,
                                                    hf2,
                                                    bin_idx,
                                                    nbin,
                                                    debug_mode,
                                                    nx,
                                                    ny,
                                                    nz)
    return bin_fsc,noisevar,signalvar,totalvar,fo,eo

def anytwomaps_fsc_covariance(f1,f2,bin_idx,nbin):
    assert f1.shape == f2.shape
    nx,ny,nz = f1.shape
    f1f2_covar,bin_fsc,bincount = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(f1,
                                                                f2,
                                                                bin_idx,
                                                                nbin,
                                                                debug_mode,
                                                                nx,ny,nz)
    return bin_fsc,f1f2_covar

def predict_fsc(hf1,hf2,bin_idx,nbin,nparticles):
    fsc,noise,signal,total,_,_ = halfmaps_fsc_variance(hf1,hf2,bin_idx,nbin)
    factors = nparticles
    print(factors)
    predicted_fsc_lst = []
    for i in range(len(nparticles)):
        predicted_fsc_lst.append(signal / (signal + noise * factors[i]))
    predicted_fsc_lst.append(2 * fsc/(1.0 + fsc))
    return predicted_fsc_lst

    
    





"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from timeit import default_timer as timer
from emda.plotter import plot_nlines
from emda.quaternions import get_RM, rotationMatrixToEulerAngles
from emda.mapfit.utils import get_FRS,create_xyz_grid,get_xyz_sum
import fcodes_fast
from emda.restools import cut_resolution_for_linefit
from emda.mapfit.derivatives_newmethod2 import derivatives,new_derivatives,optimized_derivative_calc
from emda.config import *
from emda.mapfit import linefit_class
from emda import fsc

np.set_printoptions(suppress=True) # Suppress insignificant values for clarity

#debug_mode = 0 # 0: no debug info, 1: debug

class EmFit:
    def __init__(self,mapobj,interp='linear',dfs=None):
        self.mapobj         = mapobj
        self.cut_dim        = mapobj.cdim
        self.ful_dim        = mapobj.map_dim
        self.cell           = mapobj.map_unit_cell
        self.origin         = mapobj.map_origin
        self.interp         = interp
        self.dfs            = dfs
        self.w_grid         = None
        self.fsc            = None 
        self.sv             = None
        self.t              = None
        self.st             = None
        self.step           = None
        self.rotmat         = None

    def get_wght(self,e0,ert):
        from emda import fsc
        cx,cy,cz = e0.shape
        start = timer()
        fsc,_ = fsc.anytwomaps_fsc_covariance(e0,
                                              ert,
                                              self.mapobj.cbin_idx,
                                              self.mapobj.cbin)
        w_grid = fcodes_fast.read_into_grid(self.mapobj.cbin_idx,
                                fsc/(1-fsc**2),
                                self.mapobj.cbin,
                                cx,cy,cz)
        fsc_sqd = fsc**2
        fsc_combi = fsc_sqd/(1 - fsc_sqd)
        w2_grid = fcodes_fast.read_into_grid(self.mapobj.cbin_idx,
                                fsc_combi,
                                self.mapobj.cbin,cx,cy,cz)
        end = timer()
        print('     weight calc time: ', end-start)
        return w_grid,w2_grid,fsc

    def functional(self,e0,e1,f1=None):
        start = timer()
        cx,cy,cz = e0.shape
        self.st,s1,s2,s3 = fcodes_fast.get_st(cx,cy,cz,self.t)
        #self.st = np.transpose(st_transpose)
        self.sv = np.array([s1,s2,s3])
        self.ert = get_FRS(self.rotmat,e1 * self.st,interp=self.interp)[:,:,:,0]
        if f1 is not None:
            # translate and then rotate
            self.frt = get_FRS(self.rotmat,f1 * self.st,interp=self.interp)[:,:,:,0]
        self.w_grid,self.w2_grid,self.fsc = self.get_wght(e0, self.ert) 
        fval = np.sum(self.w_grid * e0 * np.conjugate(self.ert))
        end = timer()
        print(' functional calc time: ', end-start)
        return fval.real

    def functional_tmp(self,t,q,e0,e1):
        cx,cy,cz = e0.shape
        st,_,_,_ = fcodes_fast.get_st(cx,cy,cz,t)
        rotmat = get_RM(q)
        #self.st = np.transpose(st_transpose)
        # translate and then rotate
        ert = get_FRS(rotmat,e1 * st, interp=self.interp)[:,:,:,0]
        w_grid,_,_ = self.get_wght(e0, ert) 
        fval = np.sum(w_grid * e0 * np.conjugate(ert))
        return fval.real

    def f(self,k):
        from emda.mapfit.utils import get_fsc_wght
        # w = 1.0 for line search
        nx,ny,nz = self.e0_lf.shape
        t = self.step[:3]*k[0]
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        q_init = np.array([1.0, 0.0, 0.0, 0.0])
        tmp = np.insert(self.step[3:]*k[1], 0, 0.0)
        tmp = tmp + q_init
        q = tmp/np.sqrt(np.dot(tmp, tmp))
        rotmat = get_RM(q)
        ers = get_FRS(rotmat,self.e1_lf * st, interp=self.interp)
        w_grid = get_fsc_wght(self.e0_lf, ers[:,:,:,0],self.cbin_idx_lf,self.cbin_lf)
        fval = np.sum(w_grid * self.e0_lf * np.conjugate(ers[:,:,:,0]))
        return -fval.real

    def calc_fval_for_different_kvalues_at_this_step(self,step,e0,e1,smax):
        from scipy import optimize
        nx,ny,nz = e0.shape
        w = 1.0
        start = timer()
        self.e0_lf,self.cbin_idx_lf,self.cbin_lf = cut_resolution_for_linefit(self.e0,
                                                    self.mapobj.cbin_idx,
                                                    self.mapobj.res_arr,
                                                    smax)
        self.e1_lf,_,_ = cut_resolution_for_linefit(self.e1,
                                                   self.mapobj.cbin_idx,
                                                   self.mapobj.res_arr,
                                                   smax)
        #self.e0_lf = e0
        #self.e1_lf = e1
        #self.cbin_idx_lf = self.mapobj.cbin_idx
        #self.cbin_lf = self.mapobj.cbin
        init_guess = [1.0,1.0]
        minimum = optimize.minimize(self.f, init_guess, method='Powell')
        end = timer()
        #print('time for line search: ', end-start)
        return minimum.x

    def minimizer(self,ncycles,t_init,rotmat,smax_lf,fobj=None):
        from emda.mapfit import rotmat2quart
        import math
        from emda.mapfit import interp_derivatives
        from emda.mapfit import utils
        from emda import iotools

        fsc_lst         = []
        # test
        print('smax_lf: ', smax_lf)
        #
        nfit = len(self.mapobj.ceo_lst) - 1
        self.e0 = self.mapobj.ceo_lst[0] # Static map e-data for fit
        fobj.write('\n')
        fobj.write('Normalized Structure Factors are used for fitting! \n')
        xyz = create_xyz_grid(self.cell, self.cut_dim)
        vol = self.cell[0] * self.cell[1] * self.cell[2]
        xyz_sum = get_xyz_sum(xyz)
        q_init = np.array([1.0, 0.0, 0.0, 0.0], dtype=np.float64)
        self.trans_para = []
        self.rot_para   = []
        fobj.write('\n')
        fobj.write('Cycle#\n')
        fobj.write('Function value\n')
        fobj.write('Rotation(degrees)\n')
        fobj.write('Translation(A)\n')
        fobj.write('\n')
        print('Cycle#   ', 'Func. value  ', 'Rotation(degrees)  ', 'Translation(A)  ')
        for ifit in range(nfit):
            self.e1 = self.mapobj.ceo_lst[ifit + 1]
            for i in range(ncycles):
                start = timer()
                if i == 0:
                    self.t = np.asarray(t_init)
                    t_accum = self.t
                    t_accum_angstrom = t_accum * self.cell[:3]
                    translation_vec = np.sqrt(np.sum(t_accum_angstrom * t_accum_angstrom))
                    self.rotmat = rotmat
                    q = rotmat2quart.rot2quart(self.rotmat)
                    self.q = q
                    q_accum = self.q
                    theta2 = np.arccos((np.trace(self.rotmat) - 1)/2) * 180./np.pi
                    Euler_angles = rotationMatrixToEulerAngles(self.rotmat) * 180./np.pi
                else:
                    rm_accum = get_RM(q_accum)
                    theta2 = np.arccos((np.trace(rm_accum) - 1)/2) * 180./np.pi
                    Euler_angles = rotationMatrixToEulerAngles(rm_accum) * 180./np.pi
                    if theta2 < 0.01:
                        self.rotmat = np.identity(3)
                    else:
                        self.rotmat = get_RM(self.q)
                    ## output map in each cycle for testing
                    #nx,ny,nz = self.f1.shape
                    #st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t_accum)
                    #frt_full = utils.get_FRS(rotmat,self.f1 * st, interp='cubic')[:,:,:,0]
                    #data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
                    #iotools.write_mrc(data2write,"{0}_{1}.{2}".format('fitted_map',str(i),'mrc'),self.cell,self.origin)
                    ##

                fval = self.functional(self.e0, self.e1)
                if math.isnan(theta2):
                    print('Cannot find a solution! Stopping now...')
                    exit()
                if i == 0: 
                    fval_previous = fval
                    fsc = self.fsc
                    fsc_lst.append(fsc)
                if i > 0 and fval_previous < fval or i == ncycles-1:
                    fsc = self.fsc
                if i > 0 and fval < fval_previous or i == ncycles-1:
                    rotmat = get_RM(q_accum)
                    fsc_lst.append(fsc)
                    self.rotmat = rotmat
                    self.t_accum = t_accum
                    self.fsc_lst = fsc_lst
                    break
                #print('Euler angles: [degrees]: ', Euler_angles)
                print("{:5d} {:8.4f} {:6.2f} {:6.2f}".format(
                            i,
                            fval,
                            theta2,
                            translation_vec))
                fobj.write("{:5d} {:8.4f} {:6.2f} {:6.2f}\n".format(
                            i,
                            fval,
                            theta2,
                            translation_vec))

                '''self.step,self.grad,self.e1 = derivatives(self.e0,
                            self.ert,self.w_grid,self.w2_grid,q,self.sv,xyz,xyz_sum,vol)
                self.step,self.grad,self.e1 = optimized_derivative_calc(self.e0,
                            self.ert,self.w_grid,self.w2_grid,q,self.sv,xyz,xyz_sum,vol)'''
                if self.dfs is not None:
                    start1 = timer()
                    dFRs = interp_derivatives.interp_derivatives(self.dfs,self.rotmat)
                    end1 = timer()
                    print(' time for dFRs calculation: ', end1-start1)
                elif self.dfs is None:
                    dFRs = None 
                
                self.step,self.grad = new_derivatives(self.e0,
                            self.ert,self.w_grid,self.w2_grid,q,self.sv,xyz,xyz_sum,vol,dFRs)
                self.e1 = self.ert
                start_lf = timer()
                if i == 0:
                    linefit = linefit_class.LineFit()
                    linefit.get_linefit_static_data(self.e0,
                                                    self.mapobj.cbin_idx,
                                                    self.mapobj.res_arr,
                                                    smax_lf)
                linefit.step = self.step
                #alpha = self.calc_fval_for_different_kvalues_at_this_step(self.step,self.e0,self.e1,smax_lf)
                alpha = linefit.calc_fval_for_different_kvalues_at_this_step(self.e1)
                end_lf = timer()
                print(' time for line fit: ', end_lf-start_lf)        
                # translation
                self.t = self.step[:3]*alpha[0]
                '''t_accum_angstrom = (t_accum + self.t) * self.cell[:3]
                if np.sqrt(np.dot(t_accum_angstrom, t_accum_angstrom)) < 0.01:
                    self.t = np.asarray([0.0, 0.0, 0.0])'''
                t_accum = t_accum + self.t
                t_accum_angstrom = t_accum * self.cell[:3]
                translation_vec = np.sqrt(np.sum(t_accum_angstrom * t_accum_angstrom))
                # rotation
                tmp = np.insert(self.step[3:]*alpha[1],0,0.0)
                q_accum = q_accum + tmp
                q_accum = q_accum/np.sqrt(np.dot(q_accum, q_accum))
                rm_accum = get_RM(q_accum)
                theta2 = np.arccos((np.trace(rm_accum) - 1)/2) * 180./np.pi
                if theta2 < 0.01:
                    tmp = q_init # revert to no-rotation
                else:
                    tmp = tmp + q_init
                q = tmp/np.sqrt(np.dot(tmp, tmp))
                self.q = q
                fval_previous = fval
                end = timer()
                print('time for one cycle:', end-start)
            #plot_nlines(self.mapobj.cres_arr,fsc_lst,'before_and_after_fit.eps',["Start","End"])
            






"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from timeit import default_timer as timer
from emda.mapfit import map_Class
from emda.mapfit import emfit_Class
from emda.mapfit import linefit_class
from emda.config import *

def output_rotated_maps(emmap1, r_lst, t_lst, Bf_arr=[0.0]):
    import numpy as np
    import fcodes_fast
    from emda.mapfit import utils
    from emda.plotter import plot_nlines,plot_nlines_log
    from scipy.ndimage.interpolation import shift
    from emda import iotools 
    import numpy.ma as ma
    from emda import fsc

    com = emmap1.com1  
    arr_lst = emmap1.arr_lst
    fo_lst = emmap1.fo_lst   
    bin_idx = emmap1.bin_idx  
    nbin = emmap1.nbin     
    res_arr = emmap1.res_arr  
    cell = emmap1.map_unit_cell 
    origin = emmap1.map_origin
    #assert len(fo_lst) == len(t_lst) == len(r_lst)
    nx,ny,nz = fo_lst[0].shape
    frt_lst = []
    frt_lst.append(fo_lst[0])  
    cov_lst = []
    fsc12_lst = []
    fsc12_lst_unaligned = []
    imap_f = 0
    # static map
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo_lst[0]))))
    #data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
    iotools.write_mrc(data2write,'static_map.mrc',cell,origin)
    iotools.write_mrc(arr_lst[0],'static_map_realsp.mrc',cell,origin)
    del data2write
    for fo, t, rotmat in zip(fo_lst[1:], t_lst, r_lst):
        f1f2_fsc_unaligned,_ = fsc.anytwomaps_fsc_covariance(fo_lst[0],fo,bin_idx,nbin)
        fsc12_lst_unaligned.append(f1f2_fsc_unaligned)            
        imap_f = imap_f + 1
        '''# rotate original map
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        arr = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo * st))))
        #arr = np.fft.fftshift(arr)
        print('before triliner map')
        nx, ny, nz = arr.shape
        data2write = fcodes_fast.trilinear_map(rotmat,arr,nx,ny,nz)
        print('after triliner map')'''
        #
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        frt_full = utils.get_FRS(rotmat,fo * st, interp='cubic')[:,:,:,0]
        frt_lst.append(frt_full)
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        #data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        iotools.write_mrc(data2write,"{0}_{1}.{2}".format('fitted_map',str(imap_f),'mrc'),cell,origin)
        #exit()
        # estimating covaraince between current map vs. static map
        f1f2_fsc,f1f2_covar = fsc.anytwomaps_fsc_covariance(fo_lst[0],frt_full,bin_idx,nbin)
        cov_lst.append(f1f2_covar)
        fsc12_lst.append(f1f2_fsc)

    plot_nlines(res_arr,
                fsc12_lst_unaligned + fsc12_lst,
                'fsc.eps',
                ["start FSC","end FSC"])

'''def fsc_between_static_and_transfomed_map(staticmap,movingmap,bin_idx,rm,t,cell,nbin):
    import fcodes_fast
    from emda.mapfit import utils
    from emda import fsc
    nx, ny, nz = staticmap.shape
    st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
    frt_full = utils.get_FRS(rm,movingmap * st, interp='cubic')[:,:,:,0]
    f1f2_fsc,_ = fsc.anytwomaps_fsc_covariance(staticmap,
                                               frt_full,
                                               bin_idx,
                                               nbin)
    return f1f2_fsc

def get_ibin(bin_fsc):
    import numpy as np
    bin_fsc = bin_fsc[bin_fsc > 0.1]
    #dist = np.sqrt((bin_fsc - 0.143)**2)
    dist = np.sqrt((bin_fsc - 0.4)**2)
    ibin = np.argmin(dist) + 1 # adding 1 because zero indexing
    if ibin % 2 != 0: ibin = ibin - 1
    ibin = min([len(dist), ibin])
    return ibin'''

def main(maplist, ncycles, t_init, theta_init, smax, masklist, fobj, interp):
    from emda.quaternions import get_quaternion, get_RM
    import numpy as np
    from emda.mapfit.frequency_marching import frequency_marching
    from emda.mapfit.utils import get_FRS,create_xyz_grid,get_xyz_sum
    from emda.mapfit import interp_derivatives
    from emda import scale_maps
    from emda.iotools import write_mrc
    from emda.mapfit import run_fit
    if len(maplist) < 2: 
        print(" At least 2 maps required!")
        exit()
    try:
        emmap1 = map_Class.EmmapOverlay(maplist,masklist)
        fobj.write('Map overlay\n')
    except NameError:
        emmap1 = map_Class.EmmapOverlay(maplist)
        fobj.write('Map overlay\n')
    emmap1.load_maps(fobj)
    emmap1.calc_fsc_from_maps(fobj)
    # converting theta_init to rotmat for initial iteration
    fobj.write('\n')
    fobj.write('Initial fitting parameters:\n')
    fobj.write('    Translation: '+ str(t_init)+' \n')
    fobj.write('    Rotation: '+ str(theta_init)+' \n')
    q = get_quaternion(theta_init)
    q = q/np.sqrt(np.dot(q,q))
    print('Initial quaternion: ', q)
    rotmat = get_RM(q)
    fobj.write('    Rotation matrix: '+ str(rotmat)+' \n')
    fobj.write('\n')
    fobj.write('    # fitting cycles: '+ str(ncycles)+' \n')
    t = t_init
    rotmat_lst = []
    transl_lst = []
    # resolution estimate for line-fit
    dist = np.sqrt((emmap1.res_arr - smax)**2) # smax can be a reasonable value
    slf = np.argmin(dist) + 1 
    if slf % 2 != 0: slf = slf - 1
    slf = min([len(dist), slf])
    #
    dfs_interp = False
    # preparing parameters for minimization
    if dfs_interp:
        cell = emmap1.map_unit_cell
        xyz = create_xyz_grid(cell, emmap1.map_dim)
        vol = cell[0] * cell[1] * cell[2]
    #
    for ifit in range(1, len(emmap1.eo_lst)):
        fobj.write('Fitting set#: ' + str(ifit)+' \n')
        # scale moving map to static map
        '''f0 = emmap1.fo_lst[0]
        f1 = emmap1.fo_lst[ifit]
        scale_grid = scale_maps.transfer_power(emmap1.bin_idx,
                                  emmap1.res_arr,
                                  scale_maps.scale_twomaps_by_power(f0,
                                  f1,
                                  bin_idx=emmap1.bin_idx,
                                  res_arr=emmap1.res_arr))
        f1 = f1 * scale_grid
        write_mrc(np.fft.ifftshift(np.real(np.fft.ifftn(
                                     np.fft.ifftshift(f1)))),
                                     'test_f1.mrc',
                                     emmap1.map_unit_cell,
                                     emmap1.map_origin)'''
        #
        start_fit = timer()
        if dfs_interp:
            #dfs_full = interp_derivatives.dfs_fullmap(emmap1.arr_lst[ifit],xyz,vol)
            dfs_full = interp_derivatives.dfs_fullmap(
                                     np.fft.ifftshift(
                                     np.real(np.fft.ifftn(
                                     #np.fft.ifftshift(f1)))),
                                     np.fft.ifftshift(emmap1.eo_lst[ifit])))),
                                                      xyz,vol)
        rotmat, t = run_fit.run_fit(emmap1,smax,rotmat,t,slf,ncycles,ifit,fobj,interp)
        '''for i in range(5):
            if i==0:
                smax = smax # A
                if emmap1.res_arr[0] < smax: 
                    ibin = 2
                    print('Fitting starts at ',emmap1.res_arr[ibin],' (A) instead!' )
                else:
                    dist = np.sqrt((emmap1.res_arr - smax)**2)
                    ibin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
                    if ibin % 2 != 0: ibin = ibin - 1
                    ibin = min([len(dist), ibin])
                    print('Fitting starts at ',emmap1.res_arr[ibin],' (A)' )
                ibin_old = ibin
                f1f2_fsc = fsc_between_static_and_transfomed_map(
                                            emmap1.fo_lst[0],
                                            emmap1.fo_lst[ifit],
                                            emmap1.bin_idx,
                                            rotmat,
                                            t,
                                            emmap1.map_unit_cell,
                                            emmap1.nbin)
                fsc_lst.append(f1f2_fsc)
            else:
                # Apply initial rotation and translation to calculate fsc
                f1f2_fsc = fsc_between_static_and_transfomed_map(
                                            emmap1.fo_lst[0],
                                            emmap1.fo_lst[ifit],
                                            emmap1.bin_idx,
                                            rotmat,
                                            t,
                                            emmap1.map_unit_cell,
                                            emmap1.nbin)
                ibin = get_ibin(f1f2_fsc)
                if ibin_old == ibin: 
                    fsc_lst.append(f1f2_fsc)
                    fobj.write('\n')
                    fobj.write('FSC between static and moving maps\n')
                    fobj.write('\n')
                    fobj.write('bin#\n')
                    fobj.write('resolution (A)\n')
                    fobj.write('start FSC\n')
                    fobj.write('end FSC\n')
                    fobj.write('\n')
                    for j in range(len(emmap1.res_arr)):
                        print(emmap1.res_arr[j], fsc_lst[0][j], fsc_lst[1][j])
                        fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f}\n".format(
                            j,
                            emmap1.res_arr[j],
                            fsc_lst[0][j],
                            fsc_lst[1][j]))
                    break
                else: 
                    ibin_old = ibin
            if ibin == 0:
                print('Cannot find a solution! Stopping now...')
                exit()

            static_cutmap,cBIdx, cbin = frequency_marching(emmap1.eo_lst[0],
                                            emmap1.bin_idx,
                                            emmap1.res_arr,
                                            bmax=ibin)  
            moving_cutmap,_, _ = frequency_marching(emmap1.eo_lst[ifit],
                                                emmap1.bin_idx,
                                                emmap1.res_arr,
                                                bmax=ibin)
            moving_cutmap_f1,_, _ = frequency_marching(emmap1.fo_lst[ifit],
                                                emmap1.bin_idx,
                                                emmap1.res_arr,
                                                bmax=ibin)
            if dfs_interp:
                # cut dfs_full for current size
                dfs = interp_derivatives.cut_dfs4interp(dfs_full,cbin)
                #
            else: 
                dfs = None
            assert static_cutmap.shape == moving_cutmap.shape
            emmap1.cfo_lst = [moving_cutmap_f1]
            emmap1.ceo_lst = [static_cutmap,moving_cutmap]
            emmap1.cbin_idx = cBIdx
            emmap1.cdim = moving_cutmap.shape
            emmap1.cbin = cbin
            fit = emfit_Class.EmFit(emmap1,interp=interp,dfs=dfs)
            if ibin < slf or slf == 0: slf = ibin
            slf = min([ibin, slf])
            fit.minimizer(ncycles, t, rotmat, smax_lf=slf, fobj=fobj)
            ncycles = ncycles # tweaking this you can change later # cycles
            t = fit.t_accum
            rotmat = fit.rotmat'''

        #rotmat_lst.append(fit.rotmat)
        #transl_lst.append(fit.t_accum)
        rotmat_lst.append(rotmat)
        transl_lst.append(t)
        end_fit = timer()
        print('time for fitting: ', end_fit-start_fit)

    output_rotated_maps(emmap1,
                        rotmat_lst,
                        transl_lst)


if (__name__ == "__main__"):
    maplist = ['/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_hf2.mrc',
           '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_hf2.mrc']
    masklist = ['/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_msk.mrc',
            '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_msk.mrc']
    main(maplist)






from __future__ import absolute_import, division, print_function, unicode_literals

def frequency_marching(fo,bin_idx,res_arr,bmax=None,fobj=None):
    import numpy as np
    from emda.restools import cut_resolution

    '''if bmax is not None: 
        cbin = bmax
        smax = res_arr[cbin]
    else:
        smax = 5.0 # Angstrom
        dist = np.sqrt((res_arr - smax)**2)
        cbin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
    if cbin%2 != 0: 
        cx = cbin + 1
        #cbin = cx
    else: cx = cbin'''
    cbin = cx = bmax
    print('cbin=', cbin)
    print('fit resolution:', res_arr[cbin])
    if fobj is not None:
        fobj.write('fit resolution: '+str(res_arr[cbin])+' (A) \n')
    print('**')
    dx = int((fo.shape[0] - 2*cx)/2)
    dy = int((fo.shape[1] - 2*cx)/2)
    dz = int((fo.shape[2] - 2*cx)/2)
    cBIdx = bin_idx[dx:dx+2*cx,dy:dy+2*cx,dz:dz+2*cx]
    cutmap = cut_resolution(fo,bin_idx,res_arr,cbin)[dx:dx+2*cx,dy:dy+2*cx,dz:dz+2*cx]
    return cutmap,cBIdx,cbin


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""
from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import argparse
import sys
import datetime

cmdl_parser = argparse.ArgumentParser(prog='emda', 
                                      usage='%(prog)s [commands] [arguments]')
subparsers = cmdl_parser.add_subparsers(dest='command')

mapinfo = subparsers.add_parser('info', 
                                description='Output basic information of map')
mapinfo.add_argument('--map',required=True, help='input map')

anyfsc = subparsers.add_parser('fsc', 
                                description='Calculates FSC between any two maps')
anyfsc.add_argument('--map1',required=True, help='input map1 map')
anyfsc.add_argument('--map2',required=True, help='input map2 map')

halffsc = subparsers.add_parser('halffsc', 
                                 description='Calculates FSC between half-maps')
halffsc.add_argument('--h1',required=True, help='input map1 map')
halffsc.add_argument('--h2',required=True, help='input map2 map')
halffsc.add_argument('--out',required=False,
                            default='table_variances.txt', help='output data table')

singlemapfsc = subparsers.add_parser('singlemapfsc', 
                                     description='Calculates FSC using neighbour average')
singlemapfsc.add_argument('--h1',required=True, help='input map1 map')

ccmask = subparsers.add_parser('ccmask', 
                                description='Generates mask based on halfmaps correlation')
ccmask.add_argument('--h1',required=True, help='input halfmap1 map')
ccmask.add_argument('--h2',required=True, help='input halfmap2 map')
ccmask.add_argument('--knl', required=False, 
                      type=int, default=10, help='Kernel size (pixels)')
ccmask.add_argument('--nrm', action='store_true', help='if True use normalized maps')

lowpass = subparsers.add_parser('lowpass',
                                description='lowpass filter to specified resolution')
lowpass.add_argument('--map',required=True, help='input map (mrc/map)')
lowpass.add_argument('--res',required=True, type=float, help='lowpass resolution (A)')

power = subparsers.add_parser('power', description='calculates power spectrum')
power.add_argument('--map',required=True, help='input map (mrc/map)')

applybfac = subparsers.add_parser('bfac', description='apply a B-factor to map')
applybfac.add_argument('--map',required=True, help='input map (mrc/map)')
applybfac.add_argument('--bfc',required=True, nargs='+', 
                        type=float, help='bfactor(s) to apply')
applybfac.add_argument('--out', action='store_true', help='write out map')

map_resol = subparsers.add_parser('resol', 
                                   description='estimates map resolution based on FSC')
map_resol.add_argument('--h1',required=True, help='input halfmap1 map')
map_resol.add_argument('--h2',required=True, help='input halfmap2 map')

half2full = subparsers.add_parser('half2full', 
                                   description='combine two halfmaps to give fullmap')
half2full.add_argument('--h1',required=True, help='input halfmap1 map')
half2full.add_argument('--h2',required=True, help='input halfmap2 map')
half2full.add_argument('--out',required=False, 
                            default='fullmap.mrc', help='output map (mrc/map)')

conv_map2mtz = subparsers.add_parser('map2mtz', description='MRC/MAP to MTZ conversion')
conv_map2mtz.add_argument('--map',required=True, help='input map (mrc/map)')
conv_map2mtz.add_argument('--out',required=False, 
                            default='map2mtz.mtz', help='output map (mtz)')

transform_map = subparsers.add_parser('transform', 
                                       description='apply a transformation to the map')
transform_map.add_argument('--map',required=True, help='input map (mrc/map)')
transform_map.add_argument('--tra',required=False, default=[0.0, 0.0, 0.0], 
                            nargs='+', type=float, help='translation vec. in Angstrom. eg 1.0 0.0 0.0')
transform_map.add_argument('--rot',required=False, default=0.0, 
                            type=float, help='rotation in deg')
transform_map.add_argument('--axr',required=False, default=[1,0,0], 
                            nargs='+', type=int, help='rotation axis')
transform_map.add_argument('--out',required=False, 
                            default='transformed.mrc', help='output map (mrc/map)')

conv_mtz2map = subparsers.add_parser('mtz2map', description='MTZ to MRC/MAP conversion')
conv_mtz2map.add_argument('--mtz',required=True, help='input map (mtz)')
conv_mtz2map.add_argument('--map',required=True, help='input map (mrc/map)')
conv_mtz2map.add_argument('--out',required=True, help='output map (mrc/map)')

resample_d = subparsers.add_parser('resample', description='resample map')
resample_d.add_argument('--map',required=True, help='input map (mrc/map)')
resample_d.add_argument('--pix',required=True, type=float, help='target pixel size (A)')
resample_d.add_argument('--dim', required=False, 
                         default=None, nargs='+', type=np.int, help='target map dim ')
resample_d.add_argument('--out',required=False, 
                         default='resampled.mrc', help='output map name')

realspc = subparsers.add_parser('rcc', description='real space correlation')
realspc.add_argument('--h1',required=True, help='input halfmap1 map')
realspc.add_argument('--h2',required=True, help='input halfmap2 map')
realspc.add_argument('--mdl', required=False, help='Input model (map/mrc/mtz/pdb)')
realspc.add_argument('--res', required=False, type=float, help='Resolution (A)')
realspc.add_argument('--msk', required=False, help='input mask (mrc/map)')
realspc.add_argument('--knl', required=False, 
                      type=int, default=5, help='Kernel size (pixels)')

fourierspc = subparsers.add_parser('fcc', description='Fourier space correlation')
fourierspc.add_argument('--h1',required=True, help='input halfmap1 map')
fourierspc.add_argument('--h2',required=True, help='input halfmap2 map')
fourierspc.add_argument('--knl', required=False, 
                         type=int, default=5, help='Kernel size (pixels)')

mapmodelfsc = subparsers.add_parser('mapmodelfsc', description='map-model correlation')
mapmodelfsc.add_argument('--h1', required=True, help='input halfmap1 map')
mapmodelfsc.add_argument('--h2', required=True, help='input halfmap2 map')
mapmodelfsc.add_argument('--mdf', required=True, help='input full atomic model')
mapmodelfsc.add_argument('--md1', required=True, help='input halfmap1 atomic model')
mapmodelfsc.add_argument('--msk', required=False, help='input mask (mrc/map)')
mapmodelfsc.add_argument('--res', required=False, type=float, help='Resolution (A)')
mapmodelfsc.add_argument('--dim', required=False, nargs='+', type=np.int, help='map dim ')

mapoverlay = subparsers.add_parser('overlay', description='overlay maps')
mapoverlay.add_argument('--map',required=True, nargs='+', 
                         type=str, help='maplist for overlay')
mapoverlay.add_argument('--msk',required=False, default=None, 
                         nargs='+', type=str, help='masklist for overlay')
mapoverlay.add_argument('--tra',required=False, default=[0.0, 0.0, 0.0], 
                         nargs='+', type=float, help='translation vec.')
mapoverlay.add_argument('--rot',required=False, default=0.0, 
                         type=float, help='rotation in deg')
mapoverlay.add_argument('--axr',required=False, default=[1,0,0], 
                         nargs='+', type=int, help='rotation axis')
mapoverlay.add_argument('--ncy',required=False, default=5, 
                         type=int, help='number of fitting cycles')
mapoverlay.add_argument('--res',required=False, default=6, 
                         type=float, help='starting fit resol. (A)')
mapoverlay.add_argument('--int',required=False, default='linear', 
                         type=str, help='interpolation method ([linear]/cubic)')

mapaverage = subparsers.add_parser('average', description='weighted average of several maps')
mapaverage.add_argument('--map',required=True, nargs='+', 
                         type=str, help='maplist to average')
mapaverage.add_argument('--msk',required=False, default=None, 
                         nargs='+', type=str, help='masklist for maps')
mapaverage.add_argument('--tra',required=False, default=[0.0, 0.0, 0.0], 
                         nargs='+', type=float, help='translation vec.')
mapaverage.add_argument('--rot',required=False, default=0.0, 
                         type=float, help='rotation in deg')
mapaverage.add_argument('--axr',required=False, default=[1,0,0], 
                         nargs='+', type=int, help='rotation axis')
mapaverage.add_argument('--ncy',required=False, default=10, 
                         type=int, help='number of fitting cycles')
mapaverage.add_argument('--res',required=False, default=6, 
                         type=float, help='starting fit resol. (A)')
mapaverage.add_argument('--int',required=False, default='linear', 
                         type=str, help='interpolation method ([linear]/cubic)')

diffmap = subparsers.add_parser('diffmap', 
                                   description='difference map using average maps')
diffmap.add_argument('--m1',required=True, help='input map1')
diffmap.add_argument('--m2',required=True, help='input map2')

applymask = subparsers.add_parser('applymask', 
                                   description='apply mask on the map')
applymask.add_argument('--map',required=True, help='map to be masked')
applymask.add_argument('--msk',required=True, help='mask to be applied')
applymask.add_argument('--out',required=False, 
                         default='mapmasked.mrc', help='output map name')

scalemap = subparsers.add_parser('scalemap', 
                                   description='scale onemap to another')
scalemap.add_argument('--m1',required=True, help='input map')
scalemap.add_argument('--m2',required=True, help='map to be scaled')
scalemap.add_argument('--out',required=False, 
                         default='scaledmap.mrc', help='output map name')

bestmap = subparsers.add_parser('bestmap', 
                                   description='calculate bestmap')
bestmap.add_argument('--h1',required=True, help='input halfmap1 map')
bestmap.add_argument('--h2',required=True, help='input halfmap2 map')
bestmap.add_argument('--knl',required=False, default=5, type=int, 
                         help='kernel radius (pixels)')
bestmap.add_argument('--mod',required=False, default=1, type=int, 
                         help='fsc type (1-resol bins, 2-local)')
bestmap.add_argument('--out',required=False, 
                         default='bestmap.mrc', help='output map name')

predfsc = subparsers.add_parser('predfsc', 
                                   description='predict FSC based on # particles')
predfsc.add_argument('--h1',required=True, help='input halfmap1 map')
predfsc.add_argument('--h2',required=True, help='input halfmap2 map')
predfsc.add_argument('--msk',required=False, help='mask map')
predfsc.add_argument('--npa',required=True, nargs='+', type=float,
                         help='n fold of particles')

refmac = subparsers.add_parser('refmac', 
                                   description='prepare data for refmac refinement')
refmac.add_argument('--h1',required=True, help='input halfmap1 map')
refmac.add_argument('--h2',required=True, help='input halfmap2 map')
refmac.add_argument('--msk',required=False, help='mask map')
refmac.add_argument('--bfc',required=False, nargs='+', type=float,
                         help='b-factor list')
refmac.add_argument('--out',required=False,
                            default='output.mtz', help='output mtz file name')

occ = subparsers.add_parser('occ', 
                                   description='overall correlation in real space')
occ.add_argument('--m1',required=True, help='input map1 map')
occ.add_argument('--m2',required=True, help='input map2 map')
occ.add_argument('--msk',required=False, help='mask map')
occ.add_argument('--spc',required=False, default='real', 
                            help='space (real/fourier) for CC calculation')

mirror = subparsers.add_parser('mirror', 
                                   description='mirror the map')
mirror.add_argument('--map',required=True, help='input map')

model2map = subparsers.add_parser('model2map', description='calculate model based map')
model2map.add_argument('--mdl', required=True, help='input atomic model')
model2map.add_argument('--res', required=True, type=float, help='Resolution (A)')
model2map.add_argument('--dim', required=True, nargs='+', type=np.int, help='map dim ')
model2map.add_argument('--cel', required=True, nargs='+', type=np.float, help='cell parameters ')
model2map.add_argument('--bfc',required=False, default=0.0, type=float,
                         help='overall b-factor')


def apply_mask(args):
    from emda.emda_methods import applymask
    applymask(args.map, args.msk, args.out)

def map_info(args):
    from emda.emda_methods import read_map
    uc,arr,origin = read_map(args.map)
    print('Unit cell: ', uc)
    print('Sampling: ', arr.shape)
    print('Pixel size: ', round(uc[0]/arr.shape[0], 3))
    print('Origin: ', origin)

def anymap_fsc(args,fobj):
    from emda.emda_methods import twomap_fsc
    from emda import plotter
    res_arr, bin_fsc = twomap_fsc(args.map1, args.map2, fobj=fobj)
    plotter.plot_nlines(res_arr,
                        [bin_fsc],
                        'twomap_fsc.eps',
                        curve_label=["twomap_fsc"])

def halfmap_fsc(args):
    from emda.emda_methods import halfmap_fsc
    from emda import plotter
    res_arr, bin_fsc = halfmap_fsc(args.h1, args.h2, args.out)
    plotter.plot_nlines(res_arr,
                        [bin_fsc],
                        'halfmap_fsc.eps',
                        curve_label=["halfmap_fsc"])

def singlemap_fsc(args):
    from emda.emda_methods import singlemap_fsc as sfsc
    from emda import plotter
    res_arr, bin_fsc = sfsc(args.h1)
    plotter.plot_nlines(res_arr,
                        [bin_fsc],
                        'map_fsc.eps',
                        curve_label=["map_fsc"])

def cc_mask(args):
    from emda.emda_methods import mask_from_halfmaps
    _ = mask_from_halfmaps(half1name=args.h1,
                           half2name=args.h2,
                           radius=args.knl,
                           norm=args.nrm)

def lowpass_map(args):
    from emda.emda_methods import lowpass_map
    lowpass_map(args.map,args.res)

def power_map(args):
    from emda.emda_methods import get_map_power
    from emda import plotter
    res_arr, power_spectrum = get_map_power(args.map)
    plotter.plot_nlines_log(res_arr,
                            [power_spectrum],
                            curve_label=["Power"],
                            mapname='map_power.eps')

def mapresol(args):
    from emda.emda_methods import estimate_map_resol
    resol = estimate_map_resol(args.h1, args.h2)
    print('Map resolution (A):', resol)

def map2mtz(args):
    from emda import emda_methods
    if args.out.endswith(('.mtz')):
        outfile = args.out
    else:
        outfile = args.out+'.mtz'
    emda_methods.map2mtz(args.map,outfile)

def mtz2map(args):
    from emda import emda_methods
    from emda.iotools import read_map,write_mrc
    uc,ar,origin = read_map(args.map)
    dat = emda_methods.mtz2map(args.mtz,ar.shape)
    if args.out.endswith(('.mrc')):
        outfile = args.out
    else:
        outfile = args.out+'.mrc'
    write_mrc(dat,outfile,uc,origin)


def resample_data(args):
    import numpy as np
    from emda.emda_methods import read_map, resample_data, write_mrc
    import emda.mapfit.utils as utils
    uc, arr, org = read_map(args.map)
    arr = utils.set_dim_even(arr)
    target_uc = uc
    if args.pix is None: 
        pix = uc[0] / arr.shape[0]
    else: 
        pix = args.pix
    if args.dim is None: 
        dim = int(round(uc[0] / pix))
        new_arr = resample_data(pix, [dim,dim,dim], target_uc, arr)
    if args.dim is not None: 
        new_arr = resample_data(pix, args.dim, target_uc, arr)
        target_uc = round(pix,3) * np.asarray(args.dim, dtype='int')
    write_mrc(new_arr,args.out,target_uc,org)

def realsp_corr(args):
    from emda.emda_methods import realsp_correlation
    realsp_correlation(args.h1, args.h2, args.knl, args.mdl, args.res, args.msk)

def fouriersp_corr(args):
    from emda.emda_methods import fouriersp_correlation
    fouriersp_correlation(args.h1, args.h2, args.knl)

def map_model_fsc(args):
    from emda.emda_methods import map_model_validate
    map_model_validate(args.h1, args.h2, args.mdf, args.md1, args.msk, args.dim, args.res)

def map_overlay(args,fobj):
    from emda.emda_methods import overlay_maps
    overlay_maps(args.map, args.msk, args.tra, args.rot, args.axr, args.ncy, args.res, fobj, args.int)

def map_transform(args):
    from emda.emda_methods import map_transform
    map_transform(args.map, args.tra, args.rot, args.axr, args.out)

def map_average(args,fobj):
    from emda.emda_methods import average_maps
    fobj.write('***** Map Average *****\n')
    average_maps(args.map, args.msk, args.tra, args.rot, args.axr, args.ncy, args.res, fobj, args.int)

def apply_bfac(args):
    from emda.emda_methods import apply_bfactor_to_map
    all_maps = apply_bfactor_to_map(args.map,args.bfc,args.out)

def half_to_full(args):
    from emda.emda_methods import half2full
    fullmap = half2full(args.h1, args.h2, args.out)

def diff_map(args):
    from emda.emda_methods import difference_map
    difference_map(args.m1, args.m2)

def scale_map(args):
    from emda.emda_methods import scale_map2map
    scaled_map = scale_map2map(args.m1, args.m2, args.out)

def best_map(args):
    from emda.emda_methods import bestmap
    bestmap(hf1name=args.h1, hf2name=args.h2, 
            outfile=args.out, mode=args.mod, 
            knl=args.knl)

def pred_fsc(args):
    from emda.emda_methods import predict_fsc
    import numpy as np
    npa = np.asarray(args.npa, dtype='float')
    fsc_lst,res_arr,bin_idx,nbin = predict_fsc(hf1name=args.h1, 
                                               hf2name=args.h2,
                                               nparticles=1.0/npa,
                                               mask=args.msk)

def refmac_data(args):
    from emda.emda_methods import prepare_refmac_data
    prepare_refmac_data(hf1name=args.h1,
                        hf2name=args.h2,
                        maskname=args.msk,
                        bfac=args.bfc,
                        outfile=args.out)

def overallcc(args):
    from emda.emda_methods import overall_cc
    occ = overall_cc(map1name=args.m1, 
                     map2name=args.m2, 
                     maskname=args.msk, 
                     space=args.spc)

def mirrormap(args):
    from emda.emda_methods import mirror_map
    mirror_map(args.map)

def modeltomap(args):
    from emda.emda_methods import model2map,write_mrc
    modelmap = model2map(modelxyz=args.mdl,
                         dim=args.dim,
                         resol=args.res,
                         bfac=args.bfc,
                         cell=args.cel)
    write_mrc(modelmap,'modelmap.mrc',args.cel)



def main(command_line=None):
    f=open("EMDA.txt", 'w')
    f.write('EMDA session recorded at %s.\n\n' % 
               (datetime.datetime.now()))
    args = cmdl_parser.parse_args(command_line)
    if args.command == 'info':
        map_info(args)    
    if args.command == 'fsc':
        anymap_fsc(args,f)
        f.close()
    if args.command == 'halffsc':
        halfmap_fsc(args)
    if args.command == 'ccmask':
        cc_mask(args)
    if args.command == 'lowpass':
        lowpass_map(args)
    if args.command == 'power':
        power_map(args)
    if args.command == 'resol':
        mapresol(args)
    if args.command == 'map2mtz':
        map2mtz(args)
    if args.command == 'mtz2map':
        mtz2map(args)
    if args.command == 'resample':
        resample_data(args)
    if args.command == 'rcc':
        realsp_corr(args)
    if args.command == 'fcc':
        fouriersp_corr(args)
    if args.command == 'mapmodelfsc':
        map_model_fsc(args)
    if args.command == 'overlay':
        map_overlay(args,f)
        f.close()
    if args.command == 'average':
        map_average(args,f)
        f.close()
    if args.command == 'transform':
        map_transform(args)
    if args.command == 'bfac':
        apply_bfac(args)
    if args.command == 'singlemapfsc':
        singlemap_fsc(args)
    if args.command == 'half2full':
        half_to_full(args)
    if args.command == 'diffmap':
        diff_map(args)
    if args.command == 'applymask':
        apply_mask(args)
    if args.command == 'scalemap':
        scale_map(args)
    if args.command == 'bestmap':
        best_map(args)
    if args.command == 'predfsc':
        pred_fsc(args)
    if args.command == 'refmac':
        refmac_data(args)
    if args.command == 'occ':
        overallcc(args)
    if args.command == 'mirror':
        mirrormap(args)
    if args.command == 'model2map':
        modeltomap(args)


if __name__ == '__main__':
    main()





"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology

This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from emda import iotools
import fcodes_fast

def main():
    iotools.test()
    fcodes_fast.test()

if __name__ == '__main__':
    main()



# derivative calculation using interpolation
def interp_derivatives(dfs,rm):
    from emda.mapfit.utils import get_interp
    dFRs = get_interp(rm,dfs[:,:,:,:],interp='cubic')
    return dFRs

def dfs_fullmap(mapin,xyz,vol):
    import numpy as np
    nx,ny,nz = mapin.shape
    dfs_full = np.zeros(shape=(nx,ny,nz,3),dtype=np.complex64)
    for i in range(3):  
        dfs_full[:,:,:,i] = np.fft.fftshift(
                                      (1/vol) * 2j * np.pi * 
                                      np.fft.fftn(mapin * xyz[i]))
    return dfs_full

def cut_dfs4interp(dfs_full,cx):
    nx = dfs_full.shape[0]
    dx = int((nx - 2*cx)/2)
    dy = int((nx - 2*cx)/2)
    dz = int((nx - 2*cx)/2)
    dfs = dfs_full[dx:dx+2*cx, dy:dy+2*cx, dz:dz+2*cx, :]
    return dfs


# Applying B-factor and calculating signal
from __future__ import absolute_import, division, print_function, unicode_literals

from emda.mapfit import mapaverage
from emda.config import *

def output_rotated_maps(emmap1, r_lst, t_lst, Bf_arr=[0.0], fobj=None):
    import numpy as np
    import fcodes_fast
    from emda.mapfit import utils
    from emda.plotter import plot_nlines,plot_nlines_log
    from scipy.ndimage.interpolation import shift
    from emda import iotools
    import numpy.ma as ma
    from emda import fsc

    com_lst = emmap1.com_lst 
    fo_lst = emmap1.fo_lst   
    eo_lst = emmap1.eo_lst
    #hffsc_lst = emmap1.hffsc_lst 
    #signalvar_lst = emmap1.signalvar_lst
    #totalvar_lst = emmap1.totalvar_lst
    hffsc_lst = []
    hffsc_lst.append(mapaverage.set_array(emmap1.hffsc_lst[0]))
    #signalvar_lst = []
    #signalvar_lst.append(emmap1.signalvar_lst[0])
    totalvar_lst = []
    totalvar_lst.append(emmap1.totalvar_lst[0])
    bin_idx = emmap1.bin_idx  
    nbin = emmap1.nbin     
    res_arr = emmap1.res_arr  
    cell = emmap1.map_unit_cell 
    origin = emmap1.map_origin
    #assert len(fo_lst) == len(t_lst) == len(r_lst)
    nx,ny,nz = fo_lst[0].shape
    frt_lst = []
    frt_lst.append(fo_lst[0])  
    ert_lst = []
    ert_lst.append(eo_lst[0])  
    cov_lst = []
    #fsc12_lst = []
    i = -1
    # static map
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo_lst[0]))))
    data2write = shift(data2write, np.subtract(com_lst[0],emmap1.box_centr))
    iotools.write_mrc(data2write,'static_map.mrc',cell,origin)
    fobj.write('\n static_map.mrc was written. \n')
    del data2write
    signalvar_lst = []
    signalvar_static = emmap1.signalvar_lst[0]
    fobj.write('\n ***Map averaging using normalized structure factors*** \n')
    # smoothening signal
    fobj.write('\nStatic map signal \n')
    signalvar_static = get_extended_signal(res_arr,
                                           mapaverage.set_array(signalvar_static),
                                           hffsc_lst[0],
                                           fobj=fobj)
    signalvar_lst.append(mapaverage.set_array(signalvar_static))
    for fo, eo, t, rotmat in zip(fo_lst[1:], eo_lst[1:], t_lst, r_lst):
        i = i + 1
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        #frt_full = utils.get_FRS(cell,rotmat,fo * st)[:,:,:,0]
        #frt_lst.append(frt_full)
        '''data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        iotools.write_mrc(data2write,
                          "{0}_{1}.{2}".format('fitted_map',str(1),'mrc'),
                          cell,
                          origin)'''

        # estimating covaraince between current map vs. static map
        '''f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full,bin_idx,nbin,0,nx,ny,nz)
        # mask f1f2_covar so that anything beyond zero get zeros
        f1f2_covar = mapaverage.set_array(f1f2_covar)
        cov_lst.append(f1f2_covar)
        fsc12_lst.append(f1f2_fsc)
        # normalised maps
        ert_lst.append(utils.get_FRS(cell,rotmat,eo * st)[:,:,:,0])'''

        ######
        # apply transformation on half maps of moving map
        fobj.write('Apply transformation on half maps... \n')
        frt_hf1 = utils.get_FRS(cell,rotmat,emmap1.fhf_lst[2*i+2] * st)[:,:,:,0]
        frt_hf2 = utils.get_FRS(cell,rotmat,emmap1.fhf_lst[2*i+3] * st)[:,:,:,0]
        #frt_hf1 = utils.get_FRS(cell,rotmat,emmap1.unmask_fhf_lst[2*i+2] * st)[:,:,:,0]
        #frt_hf2 = utils.get_FRS(cell,rotmat,emmap1.unmask_fhf_lst[2*i+3] * st)[:,:,:,0]
        frt_full = (frt_hf1 + frt_hf2)/2.0
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com_lst[0],emmap1.box_centr))
        iotools.write_mrc(data2write,
                          "{0}_{1}.{2}".format('fitted_map',str(i),'mrc'),
                          cell,
                          origin)
        '''# re-estimate sigvar and totvar
        bin_fsc,noisevar,signalvar,totalvar,frt_full,ert_full = fsc.halfmaps_fsc_variance(
                                            frt_hf1,frt_hf2,bin_idx,nbin)
        #frt_full,ert_full,noisevar,signalvar,totalvar,bin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
        #        frt_hf1,frt_hf2,bin_idx,nbin,debug_mode,nx,ny,nz) 
        ert_lst.append(ert_full)
        #frt_lst.append(frt)
        signalvar_lst.append(mapaverage.set_array(signalvar))
        totalvar_lst.append(mapaverage.set_array(totalvar))
        hffsc_lst.append(mapaverage.set_array(bin_fsc))
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        iotools.write_mrc(data2write,
                          "{0}_{1}.{2}".format('fitted_map',str(i),'mrc'),
                          cell,
                          origin)'''
        ######

        # estimating covariance between current map vs. static map
        _,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full,bin_idx,nbin,0,nx,ny,nz)
        # find the relative B-factor between static and frt_full
        resol = mapaverage.get_resol(f1f2_fsc,res_arr)
        scale, bfac_relative = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_full],cell,resol)
        fobj.write('Relative B-factor between static map and moving(full) map: '+ str(bfac_relative)+'\n')
        # apply B-factor and scale to 2nd map
        s_grid = (emmap1.s_grid**2)/4.0
        frt_full_scaled = scale * frt_full * np.exp(bfac_relative * s_grid)
        # re-estimate covariance
        f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full_scaled,bin_idx,nbin,0,nx,ny,nz)
        # smoothen signal
        fobj.write('\nCovariance between static and moving maps \n')
        f1f2_covar = get_extended_signal(res_arr,mapaverage.set_array(f1f2_covar),f1f2_fsc,fobj=fobj)
        # mask f1f2_covar so that anything beyond zero get zeros
        cov_lst.append(mapaverage.set_array(f1f2_covar))
        #fsc12_lst.append(f1f2_fsc)
        scale1, bfac_relative1 = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_hf1],cell,resol)
        scale2, bfac_relative2 = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_hf2],cell,resol)
        frt_hf1_scaled = scale1 * frt_hf1 * np.exp(bfac_relative1 * s_grid)
        frt_hf2_scaled = scale2 * frt_hf2 * np.exp(bfac_relative2 * s_grid)
        bin_fsc,noisevar,signalvar,totalvar,frt_full_scaled,ert_full = fsc.halfmaps_fsc_variance(
                                            frt_hf1_scaled,frt_hf2_scaled,bin_idx,nbin)
        #_,ert_full,noisevar,signalvar,totalvar,bin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
        #        frt_hf1_scaled,frt_hf2_scaled,bin_idx,nbin,debug_mode,nx,ny,nz)
        ert_lst.append(ert_full)
        # smoothen signal
        bin_fsc_full = 2*bin_fsc/(1+bin_fsc)
        fobj.write('\nMoving map signal \n')
        signalvar = get_extended_signal(res_arr,mapaverage.set_array(signalvar),bin_fsc_full,fobj=fobj)       
        signalvar_lst.append(mapaverage.set_array(signalvar))
        totalvar_lst.append(mapaverage.set_array(totalvar))
        hffsc_lst.append(mapaverage.set_array(bin_fsc_full))

    var_cov_lst = signalvar_lst + cov_lst
    plot_nlines_log(res_arr,
                    var_cov_lst)
    fvarcov = open("var_cov.txt", "w")
    for i in range(nbin):
        s11 = var_cov_lst[0][i]
        s22 = var_cov_lst[1][i]
        s12 = var_cov_lst[2][i]
        fvarcov.write("{:.2f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],s11, s22, s12))
    # map averaging with normalised maps
    averagemaps = utils.avg_and_diffmaps(ert_lst,
                                         cell,nbin,
                                         signalvar_lst,
                                         totalvar_lst,
                                         cov_lst,
                                         hffsc_lst,
                                         bin_idx,
                                         emmap1.s_grid,
                                         res_arr,
                                         Bf_arr)
    # calc. fsc between static map and averaged maps
    '''for i in range(2):
        _,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                            eo_lst[0],averagemaps[:,:,:,i,0],bin_idx,nbin,0,nx,ny,nz)
        fsc12_lst.append(f1f2_fsc)
    plot_nlines(res_arr,
                fsc12_lst,
                'fsc_compare.eps',
                ["static-aligned2","static-avg1","static-avg2"])     ''' 
    #utils.output_maps(averagemaps,emmap1.com1,cell,origin,Bf_arr)
    utils.output_maps(averagemaps,emmap1.com_lst,emmap1.box_centr,cell,origin,Bf_arr)

def output_rotated_maps_using_fo(emmap1, r_lst, t_lst, Bf_arr=[0.0], fobj=None):
    import numpy as np
    import fcodes_fast
    from emda.mapfit import utils
    from emda.plotter import plot_nlines,plot_nlines_log
    from scipy.ndimage.interpolation import shift
    from emda import iotools
    import numpy.ma as ma

    com_lst = emmap1.com_lst
    fo_lst = emmap1.fo_lst   
    signalvar_lst = []
    #signalvar_lst.append(emmap1.signalvar_lst[0])
    totalvar_lst = []
    totalvar_lst.append(emmap1.totalvar_lst[0])
    hffsc_lst = []
    hffsc_lst.append(mapaverage.set_array(emmap1.hffsc_lst[0]))
    bin_idx = emmap1.bin_idx  
    nbin = emmap1.nbin     
    res_arr = emmap1.res_arr  
    cell = emmap1.map_unit_cell 
    origin = emmap1.map_origin
    nx,ny,nz = fo_lst[0].shape
    frt_lst = []
    frt_lst.append(fo_lst[0])   
    cov_lst = []
    fsc12_lst = []
    i = -1
    # static map
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo_lst[0]))))
    data2write = shift(data2write, np.subtract(com_lst[0],emmap1.box_centr))
    iotools.write_mrc(data2write,'static_map.mrc',cell,origin)
    del data2write
    fobj.write('\n *** Map averaging using un-normalized structure factors *** \n')
    #new_signalvar_lst = []
    #new_signalvar_lst.append(mapaverage.set_array(signalvar_lst[0]))
    #new_totalvat_lst = []
    #new_totalvat_lst.append(mapaverage.set_array(totalvar_lst[0]))
    maps2average_lst = []
    maps2average_lst.append(fo_lst[0])
    signalvar_static = emmap1.signalvar_lst[0]
    # smoothening signal
    fobj.write('\nStatic map signal \n')
    signalvar_static = get_extended_signal(res_arr,
                                           mapaverage.set_array(signalvar_static),
                                           hffsc_lst[0],
                                           fobj=fobj)
    signalvar_lst.append(mapaverage.set_array(signalvar_static))
    for fo, t, rotmat in zip(fo_lst[1:], t_lst, r_lst):
        i = i + 1
        '''data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo))))
        data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        iotools.write_mrc(data2write,'fitted_map_before.mrc',cell,origin)
        del data2write'''
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        frt_full = utils.get_FRS(cell,rotmat,fo * st)[:,:,:,0]
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com_lst[0],emmap1.box_centr))
        iotools.write_mrc(data2write,'fitted_map_'+str(i+1)+'_.mrc',cell,origin)
        del data2write
        # estimating covaraince between current map vs. static map
        _,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full,bin_idx,nbin,0,nx,ny,nz)
        # find the relative B-factor between static and frt_full
        resol = mapaverage.get_resol(f1f2_fsc,res_arr)
        '''scale, bfac_relative = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_full],cell,resol)
        # apply B-factor and scale to 2nd map
        s_grid = (emmap1.s_grid**2)/4.0
        frt_full_scaled = scale * frt_full * np.exp(bfac_relative * s_grid)
        maps2average_lst.append(frt_full_scaled)
        # re-estimate covariance with scaled data
        f1f2_covar,_ = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full_scaled,bin_idx,nbin,0,nx,ny,nz)
        f1f2_covar = mapaverage.set_array(f1f2_covar)
        cov_lst.append(f1f2_covar)'''
        ######
        # apply transformation on half maps of moving map
        '''frt_hf1 = utils.get_FRS(cell,rotmat,emmap1.unmask_fhf_lst[2*i+2] * st)[:,:,:,0]
        frt_hf2 = utils.get_FRS(cell,rotmat,emmap1.unmask_fhf_lst[2*i+3] * st)[:,:,:,0]'''
        frt_hf1 = utils.get_FRS(cell,rotmat,emmap1.fhf_lst[2*i+2] * st)[:,:,:,0]
        frt_hf2 = utils.get_FRS(cell,rotmat,emmap1.fhf_lst[2*i+3] * st)[:,:,:,0]
        s_grid = (emmap1.s_grid**2)/4.0
        scale1, bfac_relative1 = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_hf1],cell,resol)
        scale2, bfac_relative2 = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_hf2],cell,resol)
        frt_hf1_scaled = scale1 * frt_hf1 * np.exp(bfac_relative1 * s_grid)
        frt_hf2_scaled = scale2 * frt_hf2 * np.exp(bfac_relative2 * s_grid)
        # re-estimate sigvar and totvar
        bin_fsc,noisevar,signalvar,totalvar,frt_full_scaled,_ = fsc.halfmaps_fsc_variance(
                                            frt_hf1_scaled,frt_hf2_scaled,bin_idx,nbin)
        #frt_full_scaled,_,noisevar,signalvar,totalvar,bin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
        #        frt_hf1_scaled,frt_hf2_scaled,bin_idx,nbin,debug_mode,nx,ny,nz) 
        maps2average_lst.append(frt_full_scaled)
        #data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full_scaled))))
        #data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        #iotools.write_mrc(data2write,'fitted_map_scaled.mrc',cell,origin)
        #del data2write
        # smoothen signal
        bin_fsc_full = 2*bin_fsc/(1+bin_fsc)
        fobj.write('\nMoving map signal \n')
        signalvar = get_extended_signal(res_arr,mapaverage.set_array(signalvar),bin_fsc_full,fobj=fobj)       
        signalvar_lst.append(mapaverage.set_array(signalvar))

        f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full_scaled,bin_idx,nbin,0,nx,ny,nz)
        #f1f2_covar = mapaverage.set_array(f1f2_covar)
        #cov_lst.append(f1f2_covar)
        # smoothen signal
        fobj.write('\nCovariance between static and moving maps \n')
        f1f2_covar = get_extended_signal(res_arr,mapaverage.set_array(f1f2_covar),f1f2_fsc,fobj=fobj)
        # mask f1f2_covar so that anything beyond zero get zeros
        cov_lst.append(mapaverage.set_array(f1f2_covar))
        #signalvar_lst.append(mapaverage.set_array(signalvar))
        totalvar_lst.append(mapaverage.set_array(totalvar))
        ######

    var_cov_lst = signalvar_lst + cov_lst
    plot_nlines_log(res_arr,
                    var_cov_lst)
    # map averaging with un-normalised maps
    averagemaps = utils.calc_averagemaps_simple(maps2average_lst,
                                                cell, nbin,
                                                signalvar_lst,
                                                totalvar_lst,
                                                cov_lst,
                                                bin_idx,
                                                emmap1.s_grid,
                                                res_arr,
                                                Bf_arr,
                                                emmap1.com_lst,
                                                emmap1.box_centr,
                                                origin)
    # calc. fsc between static map and averaged maps
    '''for i in range(2):
        _,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                            fo_lst[0],averagemaps[:,:,:,i,0],bin_idx,nbin,0,nx,ny,nz)
        fsc12_lst.append(f1f2_fsc)
    plot_nlines(res_arr,
                fsc12_lst,
                'fsc_compare.eps',
                ["static-aligned2","static-avg1","static-avg2"])     ''' 
    #utils.output_maps(averagemaps,emmap1.com_lst,emmap1.box_centr,cell,origin,Bf_arr)
    utils.output_maps(averagemaps,emmap1.com_lst,t_lst,r_lst,emmap1.box_centr,cell,origin,Bf_arr)

def output_rotated_maps_using_fo_allmaps(emmap1, r_lst, t_lst, Bf_arr=[0.0], fobj=None):
    import numpy as np
    import fcodes_fast
    from emda.mapfit import utils
    from emda.plotter import plot_nlines,plot_nlines_log
    from scipy.ndimage.interpolation import shift
    from emda import iotools
    import numpy.ma as ma
    from emda import fsc

    com_lst = emmap1.com_lst
    fo_lst = emmap1.fo_lst 
    nmaps = len(fo_lst)  
    signalvar_lst = []
    totalvar_lst = []
    totalvar_lst.append(emmap1.totalvar_lst[0])
    hffsc_lst = []
    hffsc_lst.append(mapaverage.set_array(emmap1.hffsc_lst[0]))
    bin_idx = emmap1.bin_idx  
    nbin = emmap1.nbin     
    res_arr = emmap1.res_arr  
    cell = emmap1.map_unit_cell 
    origin = emmap1.map_origin
    nx,ny,nz = fo_lst[0].shape
    frt_lst = []
    frt_lst.append(fo_lst[0]) 
    ert_lst = []
    ert_lst.append(emmap1.eo_lst[0])  
    cov_lst = []
    #fsc12_lst = []
    i = 0
    var_covar_mat = np.zeros((nmaps, nmaps, nbin), dtype='float')
    # static map
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo_lst[0]))))
    data2write = shift(data2write, np.subtract(com_lst[0],emmap1.box_centr))
    iotools.write_mrc(data2write,'static_map.mrc',cell,origin)
    del data2write
    fobj.write('\n *** Map averaging using un-normalized structure factors *** \n')
    maps2average_lst = []
    maps2average_lst.append(fo_lst[0])
    signalvar_static = emmap1.signalvar_lst[0]
    '''# smoothening signal
    fobj.write('\nStatic map signal \n')
    signalvar_static = get_extended_signal(res_arr,
                                           mapaverage.set_array(signalvar_static),
                                           hffsc_lst[0],
                                           fobj=fobj)'''
    signalvar_lst.append(mapaverage.set_array(signalvar_static))
    var_covar_mat[0,0,:] = mapaverage.set_array(signalvar_static)
    s_grid = (emmap1.s_grid**2)/4.0

    for fo, t, rotmat in zip(fo_lst[1:], t_lst, r_lst):
        #from emda.mapfit import mapaverage
        i = i + 1
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        frt_full = utils.get_FRS(cell,rotmat,fo * st)[:,:,:,0]
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com_lst[0],emmap1.box_centr))
        iotools.write_mrc(data2write,'fitted_map_'+str(i)+'_.mrc',cell,origin)
        del data2write
        # estimating covaraince between current map vs. static map
        _,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full,bin_idx,nbin,0,nx,ny,nz)
        # get resolution from fsc
        resol = mapaverage.get_resol(f1f2_fsc,res_arr)
        ######
        # apply transformation on half maps of moving map
        '''frt_hf1 = utils.get_FRS(cell,rotmat,emmap1.unmask_fhf_lst[2*i+2] * st)[:,:,:,0]
        frt_hf2 = utils.get_FRS(cell,rotmat,emmap1.unmask_fhf_lst[2*i+3] * st)[:,:,:,0]'''
        frt_hf1 = utils.get_FRS(cell,rotmat,emmap1.fhf_lst[2*i] * st)[:,:,:,0]
        frt_hf2 = utils.get_FRS(cell,rotmat,emmap1.fhf_lst[2*i+1] * st)[:,:,:,0]
        frt_lst.append((frt_hf1+frt_hf2)/2.0)
        # find the relative B-factor between static and frt_full
        scale1, bfac_relative1 = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_hf1],cell,resol)
        scale2, bfac_relative2 = mapaverage.estimate_scale_and_relativebfac([frt_lst[0],frt_hf2],cell,resol)
        #s_grid = (emmap1.s_grid**2)/4.0
        frt_hf1_scaled = scale1 * frt_hf1 * np.exp(bfac_relative1 * s_grid)
        frt_hf2_scaled = scale2 * frt_hf2 * np.exp(bfac_relative2 * s_grid)
        # re-estimate sigvar and totvar
        bin_fsc,noisevar,signalvar,totalvar,frt_full_scaled,ert_full_scaled = fsc.halfmaps_fsc_variance(
                                            frt_hf1_scaled,frt_hf2_scaled,bin_idx,nbin)
        #frt_full_scaled,ert_full_scaled,noisevar,signalvar,totalvar,bin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
        #        frt_hf1_scaled,frt_hf2_scaled,bin_idx,nbin,debug_mode,nx,ny,nz) 
        maps2average_lst.append(frt_full_scaled)
        ert_lst.append(ert_full_scaled)
        # smoothen signal
        bin_fsc_full = 2*bin_fsc/(1+bin_fsc)
        '''fobj.write('\nMoving map signal \n')
        signalvar = get_extended_signal(res_arr,mapaverage.set_array(signalvar),bin_fsc_full,fobj=fobj) '''      
        signalvar_lst.append(mapaverage.set_array(signalvar))
        var_covar_mat[i,i,:] = mapaverage.set_array(signalvar)

        f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full_scaled,bin_idx,nbin,0,nx,ny,nz)
        # smoothen signal
        '''fobj.write('\nCovariance between static and moving maps \n')
        f1f2_covar = get_extended_signal(res_arr,mapaverage.set_array(f1f2_covar),f1f2_fsc,fobj=fobj)'''
        # mask f1f2_covar so that anything beyond zero get zeros
        cov_lst.append(mapaverage.set_array(f1f2_covar))
        var_covar_mat[0,i,:] = mapaverage.set_array(f1f2_covar)
        var_covar_mat[i,0,:] = mapaverage.set_array(f1f2_covar)
        #signalvar_lst.append(mapaverage.set_array(signalvar))
        totalvar_lst.append(mapaverage.set_array(totalvar))
        ######
    # calculating covariances between fullmaps
    fobj.write('\nCovariance between moving maps after fitting! \n')
    t_init = [0.0, 0.0, 0.0]
    theta_init = [(1,0,0), 0.0]
    ncycles = 10
    smax = 6
    for i in range(len(frt_lst)):
        for j in range(len(frt_lst)):
            if i == j: continue
            if i == 0 or j == 0: continue
            if j > i:
                frt_i = frt_lst[i]
                frt_j = frt_lst[j]
                # Can you do fitting between i and j here before calc. covariance
                rotmat, trans = mapaverage.run_fit(emmap1=emmap1,
                                   f1=ert_lst[i],#frt_i,
                                   f2=ert_lst[j],#frt_j,
                                   t_init=t_init,
                                   theta_init=theta_init,
                                   smax=smax,
                                   ncycles=ncycles,
                                   fobj=fobj)
                st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,trans)
                frt_j_new = utils.get_FRS(cell,rotmat,frt_j * st)[:,:,:,0]
                # scale 1st map to 2nd map
                _,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                        frt_i,frt_j_new,bin_idx,nbin,0,nx,ny,nz)
                # get resolution from fsc
                resol = mapaverage.get_resol(f1f2_fsc,res_arr)
                scale1, bfac_relative1 = mapaverage.estimate_scale_and_relativebfac([frt_i,frt_j_new],cell,resol)
                frt_j_scaled = scale1 * frt_j_new * np.exp(bfac_relative1 * s_grid)
                # calc covariance 
                f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                        frt_i,frt_j_scaled,bin_idx,nbin,0,nx,ny,nz)
                # smoothen signal
                fobj.write('\nCovariance between map_i and map_j maps \n')
                f1f2_covar = get_extended_signal(res_arr,mapaverage.set_array(f1f2_covar),f1f2_fsc,fobj=fobj)
                # mask f1f2_covar so that anything beyond zero get zeros
                cov_lst.append(mapaverage.set_array(f1f2_covar))
                var_covar_mat[i,j,:] = mapaverage.set_array(f1f2_covar)
            #if i > j:
            #    var_covar_mat[i,j,:]
    #
    var_cov_lst = signalvar_lst + cov_lst
    '''plot_nlines_log(res_arr,
                    var_cov_lst)'''
    # map averaging with un-normalised maps
    averagemaps = utils.calc_averagemaps_simple_allmaps(maps2average_lst,
                                                  cell, nbin,
                                                  var_covar_mat,
                                                  totalvar_lst,
                                                  bin_idx,
                                                  emmap1.s_grid,
                                                  res_arr,
                                                  Bf_arr)

    # calc. fsc between static map and averaged maps 
    utils.output_maps(averagemaps,emmap1.com_lst,t_lst,r_lst,emmap1.box_centr,cell,origin,Bf_arr)


def get_extended_signal(res_arr,signal,bin_fsc,fobj=None):
    from scipy import stats
    import numpy as np
    from emda.plotter import plot_nlines_log
    fobj.write('\nSignal extended using B-factor \n')
    fsc_mask = bin_fsc > 0.0
    bin_fsc_masked = bin_fsc[fsc_mask]
    #dist1 = np.sqrt((bin_fsc - 0.143)**2)
    dist2 = np.sqrt((bin_fsc - 0.3)**2)
    res_arr_masked = res_arr[fsc_mask]
    uplim = np.argmin(dist2)
    #lower_lim = np.argmin(dist1)
    '''if upper_lim < len(signal):
        lower_lim = upper_lim - 10
    else:
        lower_lim = 0'''
    # Need to determine the low and high limits
    from emda.mapfit import rdp_algo
    x = 1.0/res_arr_masked[:uplim]
    y = signal[:uplim]
    xc, yc = rdp_algo.run_rdp(x, np.log(y), epsilon=0.01)
    upl = np.argmin(np.sqrt((x - xc[-1])**2))
    lwl = np.argmin(np.sqrt((x - xc[-2])**2))
    #
    #s1 = 1.0/res_arr[signal > 0.0][60:75]
    s1 = x[lwl:upl]
    sig1 = y[lwl:upl]
    fobj.write('low resol. limit: '+ str(res_arr[signal > 0.0][lwl])+' (A) \n')
    fobj.write('high resol. limit: '+ str(res_arr[signal > 0.0][upl])+' (A) \n')
    #slope, intercept,_,_,_ = stats.linregress((s1*s1)/4.0, 
    #                                           np.log(signal[signal > 0.0][60:75]))
    slope, intercept,_,_,_ = stats.linregress((s1*s1)/4.0, np.log(sig1))
    bfac = slope
    print('B factor = ', bfac, 'Intercept = ', intercept)  
    fobj.write('B-factor: '+ str(bfac)+' \n')
    fobj.write('Intercept: '+ str(intercept)+' \n')
    s = 1.0/res_arr
    signal_pred = np.exp((bfac/4)*s**2 + intercept)
    new_signal = signal
    #new_signal[75:] = signal_pred[75:]
    new_signal[upl:] = signal_pred[upl:]
    '''plot_nlines_log(res_arr,
                    [signal, signal_pred, new_signal])'''
    return new_signal


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

debug_mode = 0 # 0 - no debug info; 1 - debug info



"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import fcodes_fast
from emda.plotter import plot_nlines,plot_nlines_log
from timeit import default_timer as timer
from emda.config import *

#debug_mode = 0 # 0: no debug info, 1: debug

def get_FRS(RM,E2,interp):
    # scipy interpolation  
    #ERS = interpolate_scipy(E2,RM) 
    #ERS = np.expand_dims(ERS, axis=3) 
    # end
    if len(E2.shape) == 3:
        E2 = np.expand_dims(E2, axis=3)
    ERS = get_interp(RM,E2,interp)
    return ERS

def get_interp(RM,data,interp):
    import fcodes_fast
    assert len(data.shape) == 4
    ih,ik,il,n = data.shape
    if interp == 'cubic':
        interp3d = fcodes_fast.tricubic(RM,data,debug_mode,n,ih,ik,il)
    if interp == 'linear':
        interp3d = fcodes_fast.trilinear(RM,data,debug_mode,n,ih,ik,il)
    return interp3d

def create_xyz_grid(uc,nxyz):
    x = np.fft.fftfreq(nxyz[0]) * uc[0]
    y = np.fft.fftfreq(nxyz[1]) * uc[1]
    z = np.fft.fftfreq(nxyz[2]) * uc[2]
    xv, yv, zv = np.meshgrid(x,y,z)
    xyz = [yv,xv,zv]
    for i in range(3):
        xyz[i] = np.fft.ifftshift(xyz[i])
    return xyz

def get_xyz_sum(xyz):
    xyz_sum = np.zeros(shape=(6),dtype='float')
    n = -1
    for i in range(3):  
        for j in range(3):
            if i == 0:
                sumxyz = np.sum(xyz[i] * xyz[j])
            elif i > 0 and j >= i:
                sumxyz = np.sum(xyz[i] * xyz[j])
            else:
                continue
            n = n + 1
            xyz_sum[n] = sumxyz   
    #print(xyz_sum)
    return xyz_sum

def avg_and_diffmaps(maps2avg,uc,nbin,sgnl_var,totl_var,covar,hffsc,bin_idx,s_grid,res_arr,Bf_arr):
    # average and difference map calculation
    import numpy as np
    import numpy.ma as ma
    import fcodes_fast
    nx,ny,nz = maps2avg[0].shape
    nmaps = len(maps2avg)
    unit_cell = uc
    all_maps = np.zeros(shape=(nx,ny,nz,nmaps),dtype='complex')
    for i in range(nmaps):
        all_maps[:,:,:,i] = maps2avg[i]
    print(all_maps.shape)
    #
    S_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float')
    T_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float')
    F_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float')
    # Populate Sigma matrix
    start = timer()
    for i in range(nmaps):
        for j in range(nmaps):
            if i == j:
                print('i,j=',i,j)
                # Diagonals
                S_mat[i,i,:] = 1.0
                T_mat[i,i,:] = 1.0
                F_mat[i,i,:] = np.sqrt(ma.masked_less_equal(hffsc[i],0.0).filled(0.0))
                #F_mat[i,i,:] = ma.masked_less_equal(hffsc[i],0.0).filled(0.0)
            elif i==0 and j > i:
                # Off diagonals
                print('i,j=',i,j)
                plot_nlines_log(res_arr,[covar[j-1],sgnl_var[i],sgnl_var[j]],
                                ["S12","S11","S22"],'log_variance_signal.eps')
                Sigma_ij_ma = ma.masked_less_equal(covar[j-1],0.0)
                sgnl_var_ij_ma = ma.masked_where(ma.getmask(Sigma_ij_ma), sgnl_var[i] * sgnl_var[j])
                S_mat[i,j,:] = (Sigma_ij_ma / np.sqrt(sgnl_var_ij_ma)).filled(0.0)
                Totalv_ij_ma = ma.masked_where(ma.getmask(Sigma_ij_ma), totl_var[i] * totl_var[j])
                T_mat[i,j,:] = (Sigma_ij_ma / np.sqrt(Totalv_ij_ma)).filled(0.0)
                plot_nlines_log(res_arr,
                                [covar[j-1],totl_var[i],totl_var[j]],
                                ["S12","T11","T22"],'log_variance_totalv.eps')
            elif j==0 and i > j:
                print('i,j=',i,j)
                S_mat[i,j,:] = S_mat[j,i,:]
                T_mat[i,j,:] = T_mat[j,i,:]
            #else:
            #    print('i,j=',i,j)
            #    S_mat[i,j,:] = S_mat[j,i,:]
            #    T_mat[i,j,:] = T_mat[j,i,:]
    # Plotting
    plot_nlines(res_arr,[S_mat[0,0,:],S_mat[0,1,:],S_mat[1,0,:],S_mat[1,1,:]],
                        'S_mat_fsc_ij.eps',
                        ["FSC11","FSC12","FSC21","FSC22"])
    plot_nlines(res_arr,[F_mat[0,0,:],F_mat[1,1,:]],
                        'F_mat_fsc_ij.eps',
                        ["sqrt(FSC11)","sqrt(FSC22)"])
    plot_nlines(res_arr,[T_mat[0,0,:],T_mat[0,1,:],T_mat[1,0,:],T_mat[1,1,:]],
                        'T_mat_fsc_ij.eps',
                        ["FSC11","FSC12","FSC21","FSC22"])
    end = timer()
    print('Time for sigma matrix population: ', end-start)
    start = timer()
    # Variance weighted matrices calculation
    Wgt = np.zeros(shape=(nmaps,nmaps,nbin))
    for ibin in range(nbin):
        T_mat_inv = np.linalg.pinv(T_mat[:,:,ibin]) # Moore-Penrose psedo-inversion
        tmp = np.dot(F_mat[:,:,ibin],T_mat_inv)
        Wgt[:,:,ibin] = np.dot(S_mat[:,:,ibin],tmp)
    plot_nlines(res_arr,[Wgt[0,0,:],Wgt[0,1,:],Wgt[1,0,:],Wgt[1,1,:]],
                        'Wgt_map_ij.eps',
                        ["W11","W12","W21","W22"])
    end = timer()
    print('time for pinv  ',end-start)
    # output data
    fsmat = open("smat.txt", "w")
    ftmat = open("tmat.txt", "w")
    #ftmatinv = open("tmatinv.txt", "w")
    fwmat = open("wmat.txt", "w")
    for i in range(nbin):
        fsmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],S_mat[0,0,i],S_mat[0,1,i],S_mat[1,0,i],S_mat[1,1,i]))
        ftmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],T_mat[0,0,i],T_mat[0,1,i],T_mat[1,0,i],T_mat[1,1,i]))
        #ftmatinv.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}".format(res_arr[i],T_mat_inv[0,0,i],T_mat_inv[0,1,i],T_mat_inv[1,0,i],T_mat_inv[1,1,i]))
        fwmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],Wgt[0,0,i],Wgt[0,1,i],Wgt[1,0,i],Wgt[1,1,i]))

    # Average map calculation
    nbf = len(Bf_arr)
    assert all_maps.shape[:3] == bin_idx.shape
    assert all_maps.shape[3] == nmaps
    AVG_Maps = fcodes_fast.calc_avg_maps(all_maps,
                                         bin_idx,
                                         s_grid,
                                         Wgt,
                                         Bf_arr,
                                         unit_cell,
                                         debug_mode,
                                         nbin,
                                         nmaps,
                                         nbf,
                                         nx,ny,nz)
    return AVG_Maps

def output_maps(averagemaps,com_lst,t_lst,r_lst,center,unit_cell,map_origin,bf_arr):
    import emda.iotools as iotools
    from emda.mapfit import utils
    import numpy as np
    start = timer()
    nx, ny, nz = averagemaps.shape[:3]
    #center = (nx/2, ny/2, nz/2)
    for ibf in range(averagemaps.shape[4]):
        if bf_arr[ibf] < 0.0:
            Bcode = '_blur'+str(abs(bf_arr[ibf]))
        elif bf_arr[ibf] > 0.0:
            Bcode = '_sharp'+str(abs(bf_arr[ibf]))
        elif bf_arr[ibf] == 0.0:
            Bcode = '_unsharpened'
        for imap in range(averagemaps.shape[3]):
            filename_mrc = 'avgmap_'+str(imap)+Bcode+'.mrc'
            com = com_lst[imap]
            if imap > 0:
                rotmat = r_lst[imap-1]
                nx,ny,nz = averagemaps[:,:,:,imap,ibf].shape
                t = -np.asarray(t_lst[imap-1], dtype='float') # inverse translation
                st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
                avgmap = utils.get_FRS(unit_cell,np.transpose(rotmat),averagemaps[:,:,:,imap,ibf],interp='cubic')[:,:,:,0]
                data2write = apply_shift(avgmap * st,com,center)
            if imap == 0: data2write = apply_shift(averagemaps[:,:,:,imap,ibf],com,center)
            iotools.write_mrc(data2write,
                              filename_mrc,
                              unit_cell,
                              map_origin)
            # Writing to .mtz file
            '''filename_mtz = 'avgmap_'+str(imap)+Bcode+'.mtz'
            write_3d2mtz_fortran(unit_cell,average_maps[:,:,:,imap,ibf],filename_mtz)'''
        
        ## Difference and Biase removed map calculation
        #nmaps = averagemaps.shape[3]
        #for m in range(nmaps-1):
        #    for n in range(m+1,nmaps):
        #        fname_diff1 = 'diffmap_m'+str(n)+'-m'+str(m)+Bcode
        #        com = com_lst[n]
        #        dm1 = apply_shift(averagemaps[:,:,:,n,ibf] - averagemaps[:,:,:,m,ibf],com,center)
        #        iotools.write_mrc(dm1,
        #                          fname_diff1+'.mrc',
        #                          unit_cell,map_origin)
        #        '''write_3d2mtz_fortran(unit_cell,dm1,fname_diff1+'.mtz')'''
        #        dm2 = apply_shift(averagemaps[:,:,:,m,ibf] - averagemaps[:,:,:,n,ibf],com,center)
        #        fname_diff2 = 'diffmap_m'+str(m)+'-m'+str(n)+Bcode
        #        iotools.write_mrc(dm2,
        #                          fname_diff2+'.mrc',
        #                          unit_cell,map_origin)
        #        '''write_3d2mtz_fortran(unit_cell,dm2,fname_diff2+'.mtz')'''
                ## 2Fo-Fc type maps
                #twom2m1 = apply_shift(averagemaps[:,:,:,n,ibf] + dm1, com, center)
                #fname1_2FoFc = '2m'+str(n)+'-m'+str(m)
                #iotools.write_mrc(twom2m1,
                #                  fname1_2FoFc+'.mrc',
                #                  unit_cell,map_origin)
                #'''write_3d2mtz_fortran(unit_cell,twom2m1,fname1_2FoFc+'.mtz')'''
                #twom1m2 = apply_shift(averagemaps[:,:,:,m,ibf] + dm2, com, center)
                #fname2_2FoFc = '2m'+str(m)+'-m'+str(n)
                #iotools.write_mrc(twom1m2,
                #                  fname2_2FoFc+'.mrc',
                #                  unit_cell,map_origin)
                #'''write_3d2mtz_fortran(unit_cell,twom1m2,fname2_2FoFc+'.mtz')'''
    end = timer()
    print('Map output time: ',end-start)

def apply_shift(f_map, com, center):
    import numpy as np
    from scipy.ndimage.interpolation import shift
    shifted_data = np.real(np.fft.ifftshift(
                         np.fft.ifftn(
                         np.fft.ifftshift(f_map))))
    shifted_data = shift(shifted_data, np.subtract(com, center))
    return shifted_data

def remove_unwanted_corners(uc,target_dim):
    from emda.restools import get_resArr,remove_edge
    nx,ny,nz = target_dim
    fResArr = get_resArr(uc,nx)
    cut_mask = remove_edge(fResArr,fResArr[-1])
    cc_mask = np.zeros(shape=(nx,ny,nz),dtype='int')
    cx, cy, cz = cut_mask.shape
    dx = (nx - cx)//2
    dy = (ny - cy)//2
    dz = (nz - cz)//2
    print(dx,dy,dz)
    cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = cut_mask
    return cc_mask

def double_the_axes(arr1):
    nx, ny, nz = arr1.shape
    big_arr1 = np.zeros((2*nx, 2*ny, 2*nz), dtype='float')
    dx = int(nx/2); dy = int(ny/2); dz = int(nz/2)
    big_arr1[dx:dx+nx, dy:dy+ny, dz:dz+nz] = arr1
    return big_arr1

def get_minimum_bounding_dims(arr1):
    # box dim check begin
    indices = np.indices(arr1.shape)
    from scipy import ndimage
    map_sum = np.sum(arr1)/1000
    arr1 = arr1 * (arr1 > map_sum)
    com = ndimage.measurements.center_of_mass(arr1)
    dist2 = ((indices[0,:,:] - com[0]) ** 2 + 
             (indices[1,:,:] - com[1]) ** 2 + 
             (indices[2,:,:] - com[2]) ** 2)
    #np.max(dist2[mask > 0.5])
    min_dim = int(np.sqrt(np.max(dist2)) * 2) + 1
    print(com,min_dim)
    # box dim check end
    return com, min_dim

def realsp_map_interpolation(arr1,RM):
    from scipy import ndimage
    com = ndimage.measurements.center_of_mass(arr1)
    print(com)
    com = (120, 120, 120)
    nx, ny, nz = arr1.shape
    print(nx, ny, nz)
    rotated_map = fcodes_fast.trilinear_map(RM,com,arr1,nx,ny,nz)
    return rotated_map

def calc_averagemaps_simple(maps2avg,uc,nbin,sgnl_var,totl_var,covar,bin_idx,s_grid,res_arr,Bf_arr,com_lst,box_centr,origin):
    # average and difference map calculation
    import numpy as np
    import numpy.ma as ma
    import fcodes_fast
    nx,ny,nz = maps2avg[0].shape
    nmaps = len(maps2avg)
    unit_cell = uc
    all_maps = np.zeros(shape=(nx,ny,nz,nmaps),dtype='complex')
    for i in range(nmaps):
        all_maps[:,:,:,i] = maps2avg[i]
    print(all_maps.shape)
    #
    S_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float') # signal variance
    T_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float') # total variance
    # Populate Sigma matrix
    start = timer()
    for i in range(nmaps):
        for j in range(nmaps):
            if i == j:
                print('i,j=',i,j)
                # Diagonals
                S_mat[i,i,:] = sgnl_var[i]
                T_mat[i,i,:] = totl_var[i]               
            elif i==0 and j > i:
                # Off diagonals
                print('i,j=',i,j)
                S_mat[i,j,:] = covar[j-1]
                T_mat[i,j,:] = covar[j-1]
            elif j==0 and i > j:
                print('i,j=',i,j)
                S_mat[i,j,:] = S_mat[j,i,:]
                T_mat[i,j,:] = T_mat[j,i,:]
            #else:
            #    print('i,j=',i,j)
            #    S_mat[i,j,:] = S_mat[j,i,:]
            #    T_mat[i,j,:] = T_mat[j,i,:]
    # Plotting
    plot_nlines_log(res_arr,[S_mat[0,0,:],S_mat[0,1,:],S_mat[1,0,:],S_mat[1,1,:]],
                            ["S11","S12","S21","S22"],'S_mat_fsc_ij.eps')
    plot_nlines_log(res_arr,[T_mat[0,0,:],T_mat[0,1,:],T_mat[1,0,:],T_mat[1,1,:]],
                            ["T11","T12","T21","T22"],'T_mat_fsc_ij.eps')

    end = timer()
    print('Time for sigma matrix population: ', end-start)
    start = timer()
    # Variance weighted matrices calculation
    Wgt = np.zeros(shape=(nmaps,nmaps,nbin))
    for ibin in range(nbin):
        T_mat_inv = np.linalg.pinv(T_mat[:,:,ibin]) # Moore-Penrose psedo-inversion
        Wgt[:,:,ibin] = np.dot(S_mat[:,:,ibin],T_mat_inv)
    plot_nlines(res_arr,[Wgt[0,0,:],Wgt[0,1,:],Wgt[1,0,:],Wgt[1,1,:]],
                        'Wgt_map_ij.eps',["W11","W12","W21","W22"])
    end = timer()
    print('time for pinv  ',end-start)
    # output data
    fsmat = open("smat.txt", "w")
    ftmat = open("tmat.txt", "w")
    #ftmatinv = open("tmatinv.txt", "w")
    fwmat = open("wmat.txt", "w")
    for i in range(nbin):
        fsmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],S_mat[0,0,i],S_mat[0,1,i],S_mat[1,0,i],S_mat[1,1,i]))
        ftmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],T_mat[0,0,i],T_mat[0,1,i],T_mat[1,0,i],T_mat[1,1,i]))
        #ftmatinv.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}".format(res_arr[i],T_mat_inv[0,0,i],T_mat_inv[0,1,i],T_mat_inv[1,0,i],T_mat_inv[1,1,i]))
        fwmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],Wgt[0,0,i],Wgt[0,1,i],Wgt[1,0,i],Wgt[1,1,i]))

    # Average map calculation
    nbf = len(Bf_arr)
    assert all_maps.shape[:3] == bin_idx.shape
    assert all_maps.shape[3] == nmaps
    from scipy.ndimage.interpolation import shift
    from emda import iotools
    '''for i in range(all_maps.shape[3]):
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(all_maps[:,:,:,i]))))
        data2write = shift(data2write, np.subtract(com1,box_centr))
        fname = 'allmaps_'+str(i)+'.mrc'
        iotools.write_mrc(data2write,fname,unit_cell,origin)'''
    AVG_Maps = fcodes_fast.calc_avg_maps(all_maps,
                                    bin_idx,
                                    s_grid,
                                    Wgt,
                                    Bf_arr,
                                    unit_cell,
                                    debug_mode,
                                    nbin,
                                    nmaps,
                                    nbf,
                                    nx,ny,nz)
    print('avg_map shape: ', AVG_Maps.shape)
    '''for i in range(AVG_Maps.shape[3]):
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(AVG_Maps[:,:,:,i,0]))))
        fname1 = 'avgmaps_before'+str(i)+'.mrc'
        iotools.write_mrc(data2write,fname1,unit_cell,origin)
        data2write = shift(data2write, np.subtract(com1,box_centr))
        fname = 'avgmaps_'+str(i)+'.mrc'
        iotools.write_mrc(data2write,fname,unit_cell,origin)'''
    return AVG_Maps

def calc_averagemaps_simple_allmaps(maps2avg,uc,nbin,varcovmat,totl_var,bin_idx,s_grid,res_arr,Bf_arr):
    # average and difference map calculation
    import numpy as np
    import numpy.ma as ma
    import fcodes_fast
    nx,ny,nz = maps2avg[0].shape
    nmaps = len(maps2avg)
    unit_cell = uc
    all_maps = np.zeros(shape=(nx,ny,nz,nmaps),dtype='complex')
    for i in range(nmaps):
        all_maps[:,:,:,i] = maps2avg[i]
    print(all_maps.shape)
    #
    S_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float') # signal variance
    T_mat = np.zeros(shape=(nmaps,nmaps,nbin),dtype='float') # total variance
    # Populate Sigma matrix
    start = timer()
    '''for i in range(nmaps):
        for j in range(nmaps):
            if i == j:
                print('i,j=',i,j)
                # Diagonals
                S_mat[i,i,:] = varcovmat[i,i,:]#sgnl_var[i]
                T_mat[i,i,:] = totl_var[i]               
            elif i==0 and j > i:
                # Off diagonals
                print('i,j=',i,j)
                S_mat[i,j,:] = varcovmat[0,j,:]#covar[j-1]
                T_mat[i,j,:] = varcovmat[0,j,:]#covar[j-1]
            elif j==0 and i > j:
                print('i,j=',i,j)
                S_mat[i,j,:] = varcovmat[0,i,:]#S_mat[j,i,:]
                T_mat[i,j,:] = varcovmat[0,i,:]#T_mat[j,i,:]
            #else:
            #    print('i,j=',i,j)
            #    S_mat[i,j,:] = S_mat[j,i,:]
            #    T_mat[i,j,:] = T_mat[j,i,:]'''
    for i in range(nmaps):
        for j in range(nmaps):
            if i == j:
                print('i,j=',i,j)
                # Diagonals
                S_mat[i,i,:] = varcovmat[i,i,:]#sgnl_var[i]
                T_mat[i,i,:] = totl_var[i]               
            elif j > i:
                # Off diagonals
                print('i,j=',i,j)
                S_mat[i,j,:] = varcovmat[i,j,:]#covar[j-1]
                T_mat[i,j,:] = varcovmat[i,j,:]#covar[j-1]
            elif i > j:
                print('i,j=',i,j)
                S_mat[i,j,:] = varcovmat[j,i,:]#S_mat[j,i,:]
                T_mat[i,j,:] = varcovmat[j,i,:]#T_mat[j,i,:]
    # Plotting
    plot_nlines_log(res_arr,[S_mat[0,0,:],S_mat[0,1,:],S_mat[1,0,:],S_mat[1,1,:]],
                            ["S11","S12","S21","S22"],'S_mat_fsc_ij.eps')
    plot_nlines_log(res_arr,[T_mat[0,0,:],T_mat[0,1,:],T_mat[1,0,:],T_mat[1,1,:]],
                            ["T11","T12","T21","T22"],'T_mat_fsc_ij.eps')

    end = timer()
    print('Time for sigma matrix population: ', end-start)
    start = timer()
    # Variance weighted matrices calculation
    Wgt = np.zeros(shape=(nmaps,nmaps,nbin))
    for ibin in range(nbin):
        T_mat_inv = np.linalg.pinv(T_mat[:,:,ibin]) # Moore-Penrose psedo-inversion
        Wgt[:,:,ibin] = np.dot(S_mat[:,:,ibin],T_mat_inv)
    plot_nlines(res_arr,[Wgt[0,0,:],Wgt[0,1,:],Wgt[1,0,:],Wgt[1,1,:]],
                        'Wgt_map_ij.eps',["W11","W12","W21","W22"])
    end = timer()
    print('time for pinv  ',end-start)
    # output data
    fsmat = open("smat.txt", "w")
    ftmat = open("tmat.txt", "w")
    #ftmatinv = open("tmatinv.txt", "w")
    fwmat = open("wmat.txt", "w")
    for i in range(nbin):
        fsmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],S_mat[0,0,i],S_mat[0,1,i],S_mat[1,0,i],S_mat[1,1,i]))
        ftmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],T_mat[0,0,i],T_mat[0,1,i],T_mat[1,0,i],T_mat[1,1,i]))
        #ftmatinv.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}".format(res_arr[i],T_mat_inv[0,0,i],T_mat_inv[0,1,i],T_mat_inv[1,0,i],T_mat_inv[1,1,i]))
        fwmat.write("{:.2f} {:.4f} {:.4f} {:.4f} {:.4f}\n".format(res_arr[i],Wgt[0,0,i],Wgt[0,1,i],Wgt[1,0,i],Wgt[1,1,i]))

    # Average map calculation
    nbf = len(Bf_arr)
    assert all_maps.shape[:3] == bin_idx.shape
    assert all_maps.shape[3] == nmaps
    AVG_Maps = fcodes_fast.calc_avg_maps(all_maps,
                                    bin_idx,
                                    s_grid,
                                    Wgt,
                                    Bf_arr,
                                    unit_cell,
                                    debug_mode,
                                    nbin,
                                    nmaps,
                                    nbf,
                                    nx,ny,nz)
    print('avg_map shape: ', AVG_Maps.shape)
    return AVG_Maps

def set_dim_even(x):
    import numpy as np
    # Check if dims are even
    if x.shape[0] % 2 != 0:
        xshape = list(x.shape)
        xshape[0] = xshape[0] + 1
        xshape[1] = xshape[1] + 1
        xshape[2] = xshape[2] + 1
        temp = np.zeros(xshape, x.dtype)
        temp[:-1, :-1, :-1] = x
        x = temp
    return x

def get_fsc_wght(e0,ert,bin_idx,nbin):
    import fcodes_fast
    from emda import fsc
    import numpy as np
    cx,cy,cz = e0.shape
    fsc,f1f2_covar = fsc.anytwomaps_fsc_covariance(e0,ert,bin_idx,nbin)
    '''f1f2_covar,fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(e0,
                            ert,
                            bin_idx,
                            nbin,
                            debug_mode,
                            cx,cy,cz)'''
    fsc = np.array(fsc, dtype=np.float64, copy=False)
    w_grid = fcodes_fast.read_into_grid(bin_idx,
                            fsc/(1-fsc**2),
                            nbin,
                            cx,cy,cz)
    w_grid = np.array(w_grid, dtype=np.float64, copy=False)
    return w_grid

def frac2ang(t_frac,pix_size):
    import numpy as np
    t_ang = np.asarray(t_frac, dtype='float') * pix_size
    return t_ang

def ang2frac(t_ang,pix_size):
    import numpy as np
    t_frac = np.asarray(t_ang, dtype='float') / pix_size
    return t_frac


# curve fit
''' 
Insights: 
https://stackoverflow.com/questions/16745588/least-squares-minimization-complex-numbers/20104454#20104454
Method of minimization: scipy.optimize.leastsq; Accoding to scipy documentation method os Levenberg–Marquardt'''

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from emda.config import *

def model(x, f2, s):
    import numpy as np
    k = x[0]; b = x[1]
    return k * f2 * np.exp(-b * s)

def fun(x, f1, f2, s):
    import numpy as np
    diff = (model(x, f2, s) - f1)
    z1d = np.zeros(f1.size*2, dtype = np.float64)
    z1d[0:z1d.size:2] = diff.real
    z1d[1:z1d.size:2] = diff.imag
    return z1d

def jac(x, f1, f2, s):
    import numpy as n
    J = np.zeros((f1.size*2, 2), dtype=np.float64)
    bs = x[1] * s
    ks = x[0] * s
    j0 = (np.exp(-bs) * f2)
    J[0:f1.size*2:2,0] = j0.real
    J[1:f1.size*2:2,0] = j0.imag
    j1 = (-ks * np.exp(-bs) * f2)
    J[0:f1.size*2:2,1] = j1.real
    J[1:f1.size*2:2,1] = j1.imag
    return J

def lsq(f1, f2, s, x0):
    from scipy.optimize import leastsq
    params, cov, infodict, mesg, ier = leastsq(fun, x0, Dfun=jac, args=(f1, f2, s),full_output=True)
    #print(params, mesg, ier)
    return params

def get_resolution(fhf_lst,uc):
    import fcodes_fast
    import numpy as np
    from emda import fsc, restools
    assert fhf_lst[0].shape == fhf_lst[1].shape
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,fhf_lst[0])
    bin_fsc,f1f2_covar = fsc.anytwomaps_fsc_covariance(fhf_lst[0],
                                                   fhf_lst[1],
                                                   bin_idx,nbin)
    '''f1f2_covar,bin_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(fhf_lst[0],
                                fhf_lst[1],
                                bin_idx,
                                nbin,
                                debug_mode,
                                nx,ny,nz)'''
    bin_fsc = bin_fsc[bin_fsc > 0.1]
    if len(bin_fsc) > 0: dist = np.sqrt((bin_fsc - 0.143)**2)
    resol = res_arr[np.argmin(dist)]
    return resol

def main(fhf_lst,uc,resol=5.0):
    import fcodes_fast
    import numpy as np
    assert fhf_lst[0].shape == fhf_lst[1].shape
    nx,ny,nz = fhf_lst[0].shape
    print(nx,ny,nz)
    maxbin = np.amax(np.array([nx//2,ny//2,nz//2])) 
    _, s_grid, mask = fcodes_fast.resolution_grid_full(uc,resol,1,maxbin,nx,ny,nz)
    # optimization
    from scipy.optimize import leastsq
    x0 = np.array([1., 10.])
    f1 = (fhf_lst[0]*mask).flatten()
    f2 = (fhf_lst[1]*mask).flatten()
    s = s_grid.flatten()
    s = (s**2)/4
    params, cov, infodict, mesg, ier = leastsq(fun, x0, Dfun=jac, args=(f1, f2, s),full_output=True)
    print(params, mesg, ier)
    return params

if (__name__ == "__main__"):
    from emda.iotools import read_map
    import numpy as np
    maplist = ['/Users/ranganaw/MRC/Map_superposition/testmaps/str11.map',
            '/Users/ranganaw/MRC/Map_superposition/testmaps/emda/hf1_rotated_5deg.mrc']
            #'/Users/ranganaw/MRC/Map_superposition/testmaps/apply_bfac/avgmap__sharp50.mrc']
    fhf_lst = []
    for imap in maplist:
        uc,arr,_ = read_map(imap)
        fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr))))
    resol = get_resolution(fhf_lst,uc)
    main(fhf_lst,uc,resol)